package com.gct.driver.core.net;

import com.gct.driver.core.SoftBasic;
import com.gct.driver.core.INetMessage;
//import IODriver.Core.Net.HslProtocol;
import com.gct.driver.core.IOTimeout;
import com.gct.driver.core.operate.OperateResult;
import com.gct.driver.core.operate.OperateResultExOne;
import com.gct.driver.core.log.ILogNet;
import com.gct.driver.core.Utilities;

import java.io.DataOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;

import java.net.SocketAddress;
import java.util.Date;
import java.util.UUID;


/**
 * 本系统所有网络类的基类，该类为抽象类，无法进行实例化
 */
public abstract class NetworkBase {

    // region Constructor
    /**
     * 实例化一个NetworkBase对象
     */
    public NetworkBase( )
    {
        Token = UUID.fromString("00000000-0000-0000-0000-000000000000");
    }

    // endregion

    // region Public Properties

    /**
     * 组件的日志工具，支持日志记录
     */
    public ILogNet LogNet = null;

    /**
     * 网络类的身份令牌
     */
    public UUID Token = null;

    // endregion

    // region Protect Member

    /**
     * 通讯类的核心套接字
     */
    protected Socket CoreSocket = null;

    // endregion

    // region Protect Method

    /**
     * 查网络套接字是否操作超时，需要对套接字进行封装
     * @param timeout HslTimeOut的对象
     * @throws InterruptedException InterruptedException
     * @throws IOException IOException
     */
    protected void ThreadPoolCheckTimeOut( IOTimeout timeout ){
        while (!timeout.IsSuccessful) {
            try {
                Thread.sleep(100);
            }
            catch (InterruptedException ex) {
            }
            if (new Date().getTime() - timeout.StartTime.getTime() > timeout.DelayTime) {
                // 连接超时或是验证超时
                if (!timeout.IsSuccessful) {
                    if(LogNet != null)
                        LogNet.WriteWarn(toString(), "Wait Time Out : " + timeout.DelayTime);
                    //timeout.Operator.Invoke();
                    try {
                        if(timeout.WorkSocket!=null) timeout.WorkSocket.close();
                    }
                    catch (IOException ex){

                    }
                }
                break;
            }
        }
    }

    // endregion

    // region Receive Content

    /**
     * 从套接字接收定长度的字节数组
     * @param socket 套接字
     * @param length 数据长度
     * @return 消息类对象
     */
    protected OperateResultExOne<byte[]> Receive(Socket socket, int length, int timeout ) {
        OperateResultExOne<byte[]> resultExOne = new OperateResultExOne<>();

        if (length == 0) {
            resultExOne.IsSuccess = true;
            resultExOne.Content = new byte[0];
            return resultExOne;
        }

        byte[] bytes_receive = new byte[length];
        try {
            if (timeout > 0) socket.setSoTimeout(timeout);
            InputStream input = socket.getInputStream();
            int count_receive = 0;
            while (count_receive < length) {
                count_receive += input.read(bytes_receive, count_receive, length - count_receive);
            }
        } catch (IOException ex) {
            CloseSocket(socket);
            resultExOne.Message = ex.getMessage();
            return resultExOne;
        }

        resultExOne.IsSuccess = true;
        resultExOne.Content = bytes_receive;
        return resultExOne;
    }

    /**
     * 从套接字接收定长度的字节数组
     * @param socket 套接字
     * @param length 数据长度
     * @return 消息类对象
     */
    protected OperateResultExOne<byte[]> Receive(Socket socket, int length )
    {
        return Receive(socket,length,-1);
    }


    /**
     * 从套接字接收指定长度的字节数据
     * @param socket 网络套接字
     * @param timeOut 超时时间
     * @param netMsg 消息格式
     * @param <TNetMessage> 类型
     * @return 消息类
     */
    protected <TNetMessage extends INetMessage> OperateResultExOne<byte[]> ReceiveMessage(Socket socket, int timeOut, TNetMessage netMsg) {
        IOTimeout IOTimeOut = new IOTimeout();
        IOTimeOut.WorkSocket = socket;
        IOTimeOut.DelayTime = timeOut;

        if (timeOut > 0) new Thread(() -> ThreadPoolCheckTimeOut(IOTimeOut)).start();

        // 接收指令头
        OperateResultExOne<byte[]> headResult = Receive(socket, netMsg.ProtocolHeadBytesLength());
        if (!headResult.IsSuccess) {
            IOTimeOut.IsSuccessful = true;
            return headResult;
        }

        netMsg.setHeadBytes(headResult.Content);
        int contentLength = netMsg.GetContentLengthByHeadBytes();
        if (contentLength <= 0) {
            IOTimeOut.IsSuccessful = true;
            netMsg.setContentBytes(new byte[0]);
            return headResult;
        }

        OperateResultExOne<byte[]> contentResult = Receive(socket, contentLength);
        if (!contentResult.IsSuccess) {
            IOTimeOut.IsSuccessful = true;
            return contentResult;
        }

        IOTimeOut.IsSuccessful = true;
        netMsg.setContentBytes(contentResult.Content);
        return OperateResultExOne.CreateSuccessResult( SoftBasic.SpliceTwoByteArray( headResult.Content, contentResult.Content ) );
    }

    // endregion

    // region Send Content

    /**
     * 发送一串数据到网络套接字中
     * @param socket 网络套接字
     * @param data 数据
     * @return 是否发送成功
     */
    protected OperateResult Send(Socket socket,byte[] data){
        OperateResult result = new OperateResult();
        if(data == null) {
            result.IsSuccess = true;
            return result;
        }
        try {
            DataOutputStream output = new DataOutputStream(socket.getOutputStream());
            output.write(data, 0, data.length);
        }
        catch (IOException ex)
        {
            result.Message = ex.getMessage();
            return  result;
        }

        result.IsSuccess = true;
        return  result;
    }

    // endregion

    // region Socket Connect

    /**
     * 创建一个套接字并且连接到服务器
     * @param endPoint 目标节点
     * @param timeOut 超时时间
     * @return 连接成功的标志
     */
    protected  OperateResultExOne<Socket> CreateSocketAndConnect(SocketAddress endPoint, int timeOut){
        Socket socket = new Socket();
        try {
            socket.connect(endPoint,timeOut);
            return OperateResultExOne.CreateSuccessResult(socket);
        }
        catch (IOException ex)
        {
            if(LogNet != null) LogNet.WriteException( "CreateSocketAndConnect", ex );
            CloseSocket(socket);
            return new OperateResultExOne<>(ex.getMessage());
        }
    }

    /**
     * 创建一个套接字并且连接到服务器
     * @param ipAddress ip地址
     * @param port 端口号
     * @param timeOut 超时时间
     * @return 连接成功的标志
     */
    protected  OperateResultExOne<Socket> CreateSocketAndConnect(String ipAddress,int port, int timeOut) {
        SocketAddress endPoint = new InetSocketAddress(ipAddress,port);
        return CreateSocketAndConnect(endPoint,timeOut);
    }

    /**
     * 创建一个套接字并且连接到服务器
     * @param ipAddress ip地址
     * @param port 端口号
     * @return 连接成功的标志
     */
    protected  OperateResultExOne<Socket> CreateSocketAndConnect(String ipAddress,int port) {
        SocketAddress endPoint = new InetSocketAddress(ipAddress,port);
        return CreateSocketAndConnect(endPoint,10000);
    }

    // endregion

    // region Read Stream

    /**
     * 读取流中的数据到缓存区
     * @param stream 流数据
     * @param buffer 缓冲数据
     * @return 带有成功标志的读取数据长度
     */
    protected OperateResultExOne<Integer> ReadStream(InputStream stream, byte[] buffer) {
        try {
            int read_count = stream.read(buffer, 0, buffer.length);
            return OperateResultExOne.CreateSuccessResult(read_count);
        } catch (IOException ex) {
            return new OperateResultExOne<>(ex.getMessage());
        }
    }

    // endregion

    // region Write Stream

    /**
     * 将字节流数据写入到输出流里面去
     * @param stream 字节流
     * @param buffer 缓存数据
     * @return 写入是否成功
     */
    protected OperateResult WriteStream(OutputStream stream, byte[] buffer ) {
        try {
            stream.write(buffer, 0, buffer.length);
            return OperateResult.CreateSuccessResult();
        } catch (IOException ex) {
            return new OperateResult(ex.getMessage());
        }
    }

    // endregion

    // region Token Check

    /**
     * 检查当前的头子节信息的令牌是否是正确的
     * @param headBytes 头子节数据
     * @return 令牌是验证成功
     */
    protected boolean CheckRemoteToken( byte[] headBytes )
    {
        return SoftBasic.IsByteTokenEquel( headBytes, Token );
    }

    //endregion

    // region Special Bytes Send

    /**
     * 发送字节数据并确认对方接收完成数据，如果结果异常，则结束通讯
     * @param socket 网络套接字
     * @param headCode 头指令
     * @param customer 用户指令
     * @param send 发送的数据
     * @return 是否发送成功
     */
//    protected OperateResult SendBaseAndCheckReceive( Socket socket, int headCode, int customer, byte[] send ) {
//        // 数据处理
//        send = HslProtocol.CommandBytes(headCode, customer, Token, send);
//
//        // 发送数据
//        OperateResult sendResult = Send(socket, send);
//        if (!sendResult.IsSuccess) return sendResult;
//
//        // 检查对方接收完成
//        OperateResultExOne<Long> checkResult = ReceiveLong(socket);
//        if (!checkResult.IsSuccess) return checkResult;
//
//        // 检查长度接收
//        if (checkResult.Content != send.length) {
//            try {
//                if (!socket.isClosed()) socket.close();
//            } catch (IOException ex) {
//
//            }
//            return new OperateResult(StringResources.Language.CommandLengthCheckFailed());
//        }
//
//        return checkResult;
//    }

    /**
     * [自校验] 发送字节数据并确认对方接收完成数据，如果结果异常，则结束通讯
     * @param socket 网络套接字
     * @param customer 用户指令
     * @param send 发送的数据
     * @return 是否发送成功
     */
//    protected OperateResult SendBytesAndCheckReceive( Socket socket, int customer, byte[] send )
//    {
//        return SendBaseAndCheckReceive( socket, HslProtocol.ProtocolUserBytes, customer, send );
//    }

    /**
     * [自校验] 直接发送字符串数据并确认对方接收完成数据，如果结果异常，则结束通讯
     * @param socket 网络套接字
     * @param customer 用户指令
     * @param send 发送的数据
     * @return 是否发送成功
     */
//    protected OperateResult SendStringAndCheckReceive( Socket socket, int customer, String send )
//    {
//        byte[] data = Utilities.IsStringNullOrEmpty( send ) ? null : Utilities.string2Byte( send );
//
//        return SendBaseAndCheckReceive( socket, HslProtocol.ProtocolUserString, customer, data );
//    }

    /**
     * [自校验] 直接发送字符串数组并确认对方接收完成数据，如果结果异常，则结束通讯
     * @param socket 网络套接字
     * @param customer 用户指令
     * @param sends 发送的字符串数组
     * @return 是否发送成功
     */
//    protected OperateResult SendStringAndCheckReceive( Socket socket, int customer, String[] sends )
//    {
//        return SendBaseAndCheckReceive( socket, HslProtocol.ProtocolUserStringArray, customer, HslProtocol.PackStringArrayToByte( sends ) );
//    }

    /**
     * [自校验] 直接发送字符串数组并确认对方接收完成数据，如果结果异常，则结束通讯
     * @param socket 网络套接字
     * @param customer 用户指令
     * @param name 用户名
     * @param pwd 密码
     * @return 是否发送成功
     */
//    protected OperateResult SendAccountAndCheckReceive( Socket socket, int customer, String name, String pwd )
//    {
//        return SendBaseAndCheckReceive( socket, HslProtocol.ProtocolAccountLogin, customer, HslProtocol.PackStringArrayToByte( new String[] { name, pwd } ) );
//    }

    /**
     * [自校验] 接收一条完整的同步数据，包含头子节和内容字节，基础的数据，如果结果异常，则结束通讯
     * @param socket 套接字
     * @param timeout 超时时间设置，如果为负数，则不检查超时
     * @return 包含是否成功的结果对象
     */
//    protected OperateResultExTwo<byte[], byte[]> ReceiveAndCheckBytes(Socket socket, int timeout )
//    {
//        // 30秒超时接收验证
//        HslTimeOut hslTimeOut = new HslTimeOut( );
//        hslTimeOut.DelayTime = timeout;
//        hslTimeOut.IsSuccessful = false;
//        hslTimeOut.StartTime = new Date();
//        hslTimeOut.WorkSocket = socket;
//
//        if (timeout > 0) new Thread(() -> ThreadPoolCheckTimeOut(hslTimeOut)).start();
//
//        // 接收头指令
//        OperateResultExOne<byte[]> headResult = Receive( socket, HslProtocol.HeadByteLength );
//        if (!headResult.IsSuccess)
//        {
//            hslTimeOut.IsSuccessful = true;
//            return OperateResultExTwo.CreateFailedResult( headResult );
//        }
//        hslTimeOut.IsSuccessful = true;
//
//        // 检查令牌
//        if (!CheckRemoteToken( headResult.Content ))
//        { try {
//            if (!socket.isClosed()) socket.close();
//        }
//        catch(IOException ex) {
//
//        }
//            return new OperateResultExTwo( StringResources.Language.TokenCheckFailed() );
//        }
//
//        int contentLength = Utilities.getInt( headResult.Content, HslProtocol.HeadByteLength - 4 );
//        // 接收内容
//        OperateResultExOne<byte[]> contentResult = Receive( socket, contentLength );
//        if (!contentResult.IsSuccess) return OperateResultExTwo.CreateFailedResult( contentResult );
//
//        // 返回成功信息
//        OperateResult checkResult = SendLong( socket, HslProtocol.HeadByteLength + contentLength );
//        if (!checkResult.IsSuccess) return OperateResultExTwo.CreateFailedResult( checkResult );
//
//        byte[] head = headResult.Content;
//        byte[] content = contentResult.Content;
//        content = HslProtocol.CommandAnalysis( head, content );
//        return OperateResultExTwo.CreateSuccessResult( head, content );
//    }

    /**
     * [自校验] 从网络中接收一个字符串数据，如果结果异常，则结束通讯
     * @param socket 套接字
     * @return 包含是否成功的结果对象
     */
//    protected OperateResultExTwo<Integer, String> ReceiveStringContentFromSocket( Socket socket )
//    {
//        OperateResultExTwo<byte[], byte[]> receive = ReceiveAndCheckBytes( socket, 30000 );
//        if (!receive.IsSuccess) return OperateResultExTwo.CreateFailedResult( receive );
//
//        // 检查是否是字符串信息
//        if (Utilities.getInt( receive.Content1, 0 ) != HslProtocol.ProtocolUserString)
//        {
//            if (LogNet != null) LogNet.WriteError( toString( ), StringResources.Language.CommandHeadCodeCheckFailed() );
//            try {
//                if (!socket.isClosed()) socket.close();
//            }
//            catch(IOException ex) {
//
//            }
//            return new OperateResultExTwo<>( StringResources.Language.CommandHeadCodeCheckFailed() );
//        }
//
//        if (receive.Content2 == null) receive.Content2 = new byte[0];
//        // 分析数据
//        return OperateResultExTwo.CreateSuccessResult( Utilities.getInt( receive.Content1, 4 ), Utilities.byte2String( receive.Content2 ) );
//    }

    /**
     * [自校验] 从网络中接收一个字符串数组，如果结果异常，则结束通讯
     * @param socket 套接字
     * @return 包含是否成功的结果对象
     */
//    protected OperateResultExTwo<Integer, String[]> ReceiveStringArrayContentFromSocket( Socket socket )
//    {
//        OperateResultExTwo<byte[], byte[]> receive = ReceiveAndCheckBytes( socket, 30000 );
//        if (!receive.IsSuccess) return OperateResultExTwo.CreateFailedResult( receive );
//
//        // 检查是否是字符串信息
//        if (Utilities.getInt( receive.Content1, 0 ) != HslProtocol.ProtocolUserStringArray)
//        {
//            if (LogNet != null) LogNet.WriteError( toString( ), StringResources.Language.CommandHeadCodeCheckFailed() );
//            try {
//                if (!socket.isClosed()) socket.close();
//            }
//            catch(IOException ex) {
//
//            }
//            return new OperateResultExTwo<>( StringResources.Language.CommandHeadCodeCheckFailed() );
//        }
//
//        if (receive.Content2 == null) receive.Content2 = new byte[4];
//        return OperateResultExTwo.CreateSuccessResult( Utilities.getInt( receive.Content1, 4 ), HslProtocol.UnPackStringArrayFromByte( receive.Content2 ) );
//    }

    /**
     * [自校验] 从网络中接收一串字节数据，如果结果异常，则结束通讯
     * @param socket 套接字的网络
     * @return 包含是否成功的结果对象
     */
//    protected OperateResultExTwo<Integer, byte[]> ReceiveBytesContentFromSocket( Socket socket )
//    {
//       return ReceiveBytesContentFromSocket(socket, 30000);
//    }

    /**
     * [自校验] 从网络中接收一串字节数据，如果结果异常，则结束通讯
     * @param socket 套接字的网络
     * @param timeout 超时时间
     * @return 包含是否成功的结果对象
     */
//    protected OperateResultExTwo<Integer, byte[]> ReceiveBytesContentFromSocket( Socket socket, int timeout ) {
//        OperateResultExTwo<byte[], byte[]> receive = ReceiveAndCheckBytes(socket, timeout);
//        if (!receive.IsSuccess) return OperateResultExTwo.CreateFailedResult(receive);
//
//        // 检查是否是字节信息
//        if (Utilities.getInt(receive.Content1, 0) != HslProtocol.ProtocolUserBytes) {
//            if (LogNet != null) LogNet.WriteError(toString(), StringResources.Language.CommandHeadCodeCheckFailed());
//            try {
//                if (!socket.isClosed()) socket.close();
//            }
//            catch(IOException ex) {
//
//            }
//            return new OperateResultExTwo(StringResources.Language.CommandHeadCodeCheckFailed());
//        }
//
//        // 分析数据
//        return OperateResultExTwo.CreateSuccessResult(Utilities.getInt(receive.Content1, 4), receive.Content2);
//    }

    /**
     * 从网络中接收Long数据
     * @param socket 套接字
     * @return 是否成功的类型
     */
    private OperateResultExOne<Long> ReceiveLong( Socket socket )
    {
        OperateResultExOne<byte[]> read = Receive( socket, 8 );
        if (read.IsSuccess)
        {
            return OperateResultExOne.CreateSuccessResult( Utilities.getLong(read.Content, 0) );
        }
        else
        {
            return new OperateResultExOne<Long>( read.Message );
        }
    }

    /**
     * 将long数据发送到套接字
     * @param socket 套接字
     * @param value 值
     * @return 是否成功
     */
    private OperateResult SendLong( Socket socket, long value )
    {
        return Send( socket, Utilities.getBytes( value ) );
    }

    // endregion

    // region Protect

    /**
     * 安全的关闭一个套接字
     * @param socket 网络套接字
     */
    protected void CloseSocket(Socket socket){
        if(socket != null){
            try {
                socket.close();
            }
            catch (Exception ex){

            }
        }
    }

    // endregion

    // region Object Override

    /**
     * 返回对象的字符串表示形式
     * @return 字符串
     */
    @Override
    public String toString(){
        return "NetworkBase";
    }

    // endregion
}
