package com.gct.driver.modbus;

import com.gct.driver.core.*;
import com.gct.driver.core.net.NetworkDeviceBase;
import com.gct.driver.core.operate.OperateResult;
import com.gct.driver.core.operate.OperateResultExOne;
import com.gct.driver.core.transfer.DataFormat;
import com.gct.driver.core.transfer.ReverseWordTransform;
import com.gct.driver.lotpi.ConnectionConfig;
import com.gct.driver.lotpi.DataBit;
import com.gct.driver.lotpi.DataBlock;
import com.gct.driver.lotpi.DataType;
import com.gct.driver.lotpi.exception.ConnectException;
import com.gct.driver.lotpi.exception.NotImplemented;
import com.gct.driver.lotpi.exception.UrlInvalid;
import com.gct.driver.lotpi.exception.WriteException;
import com.gct.driver.lotpi.extent.Connector;
import com.gct.driver.lotpi.message.WriteCompleted;
import com.gct.driver.lotpi.util.DriverUrlParser;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.CompletableFuture;
import java.util.function.Consumer;
import java.util.stream.Collectors;


/**
 * Modbus-Tcp协议的客户端通讯类，方便的和服务器进行数据交互
 */
public class ModbusTcp extends NetworkDeviceBase<ModbusTcpMessage, ReverseWordTransform> implements Connector {

    /**
     * 实例化一个MOdbus-Tcp协议的客户端对象
     */
    public ModbusTcp() {
        super(ModbusTcpMessage.class, ReverseWordTransform.class);
        this.softIncrementCount = new SoftIncrementCount(65535, 0);
        this.WordLength = 1;
        this.station = 1;
    }


    private byte station = 0x01;                                // 本客户端的站号
    private SoftIncrementCount softIncrementCount;              // 自增消息的对象
    private boolean isAddressStartWithZero = false;                 // 线圈值的地址值是否从零开始
    private int pageSize;
    private ModbusValueType modbusValueType = new ModbusValueType();

    /***************************************对接接口****************************************************/


    /**
     * 返回表示当前对象的字符串
     *
     * @return 字符串信息
     */
    @Override
    public String toString() {
        return "ModbusTcpNet[" + getIpAddress() + ":" + getPort() + "]";
    }


    //************************************************************
    @Override
    public void setConfig(ConnectionConfig config) {

        if (!(config instanceof ModbusTcpConfig)) throw new NotImplemented("先使用parseUrl转换");
        ModbusTcpConfig actualConfig = (ModbusTcpConfig) config;


        setIpAddress(actualConfig.getIp());
        setPort(actualConfig.getPort());

        this.station = Byte.parseByte(actualConfig.getStation());
        this.setDataFormat(DataFormat.valueOf(actualConfig.getByteOrder()));

        this.setConnectTimeOut(actualConfig.getTimeout());
        this.pageSize = actualConfig.getPageSize();

    }

    @Override
    public ConnectionConfig parseUrl(String url) {
        try {
            DriverUrlParser uri = DriverUrlParser.tryParse(url);
            ModbusTcpConfig modbusTcpConfig = new ModbusTcpConfig();
            modbusTcpConfig.setIp(uri.getHost());
            modbusTcpConfig.setPort(uri.getPort());
            modbusTcpConfig.setStation(uri.getParamByKey("station"));
            modbusTcpConfig.setTimeout(Integer.parseInt(uri.getParamByKey("timeout")));
            modbusTcpConfig.setByteOrder(Integer.parseInt(uri.getParamByKey("byteOrder")));
            String pageSize = uri.getParamByKey("pageSize");
            if (Objects.nonNull(pageSize))
                modbusTcpConfig.setPageSize(Integer.parseInt(pageSize));
            return modbusTcpConfig;
        } catch (Exception e) {
            throw new UrlInvalid("Unhandled exception: " + e.getMessage());
        }
    }

    @Override
    public boolean open() throws ConnectException {
        try {
            OperateResult res = this.ConnectServer();
            if (res.IsSuccess) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            throw new ConnectException("connect to " + getIpAddress() + ":" + getPort() + " failed");
        }
    }

    @Override
    public boolean close() throws ConnectException {
        try {
            OperateResult res = this.ConnectClose();
            if (res.IsSuccess) {
                return true;
            }
            return false;
        } catch (Exception e) {
            throw new ConnectException("connect to " + getIpAddress() + ":" + getPort() + " failed");
        }
    }

    @Override
    public boolean isOpen() {
        return !this.IsSocketError;
    }

    @Override
    public void setTimeout(int timeout) {
        setTimeout(timeout);
    }

    @Override
    public boolean preReading(DataBit[] dataBits) {
        return false;
    }

    @Override
    public DataBlock read(DataBit[] pDataBits) {
        BasicDataBit[] dataBits = BasicDataBit.transFromArray(pDataBits,modbusValueType);
        HashMap<DataBit, BasicDataBit> tempRes = new HashMap<>();
        //对传入的地址进行分类排序
        HashMap<String, ArrayList<BasicDataBit>> Zones = new HashMap<>();
        Zones.put("0", new ArrayList<>());
        Zones.put("1", new ArrayList<>());
        Zones.put("3", new ArrayList<>());
        Zones.put("4", new ArrayList<>());
        Zones.put("array", new ArrayList<>());


        for (BasicDataBit dBit : dataBits) {
            if (dBit.isArray()) {
                Zones.get("array").add(dBit);
                continue;
            }

            Zones.get(dBit.getBitAddress().toString().substring(0, 1)).add(dBit);
        }
        Zones.get("0").sort((DataBit b1, DataBit b2) -> (int) (
                Integer.parseInt(b1.getBitAddress().toString().substring(1)) - Integer.parseInt(b2.getBitAddress().toString().substring(1))
        ));
        Zones.get("1").sort((DataBit b1, DataBit b2) -> (int) (
                Integer.parseInt(b1.getBitAddress().toString().substring(1)) - Integer.parseInt(b2.getBitAddress().toString().substring(1))
        ));
        Zones.get("3").sort((DataBit b1, DataBit b2) -> (int) (

                Integer.parseInt(b1.getBitAddress().toString().substring(1)) - Integer.parseInt(b2.getBitAddress().toString().substring(1))
        ));
        Zones.get("4").sort((DataBit b1, DataBit b2) -> (int) (
                Integer.parseInt(b1.getBitAddress().toString().substring(1)) - Integer.parseInt(b2.getBitAddress().toString().substring(1))
        ));


        for (Integer z = 0; z <= 1; z++) {
            if (Zones.get(z.toString()).size() > 0) {
                String startAddr = Zones.get(z.toString()).get(0).getBitAddress().toString();
                String lasetAddr = Zones.get(z.toString()).get(0).getBitAddress().toString();
                int startIndex = 0;
                int lastIndex = -1;
                for (DataBit dbit : Zones.get(z.toString())) {
                    if (Integer.parseInt(dbit.getBitAddress().toString().substring(1)) - Integer.parseInt(lasetAddr.substring(1)) > 200) {
                        short len = (short) (Integer.parseInt(lasetAddr.substring(1)) - Integer.parseInt(startAddr.substring(1)) + 1);
                        OperateResultExOne<boolean[]> res = ReadBool(startAddr, len);
                        if (res.IsSuccess && res.Content.length == len) {
                            for (int i = startIndex; i <= lastIndex; i++) {
                                BasicDataBit temp = Zones.get(z.toString()).get(i);
                                temp.setValue(res.Content[Integer.parseInt(temp.getBitAddress().toString().substring(1)) - Integer.parseInt(startAddr.substring(1))]);
                                tempRes.put(temp.getParent(), temp);
                            }

                        } else {
                            for (int i = 0; i < Zones.get(z.toString()).size(); i++) {
                                BasicDataBit temp = Zones.get(z.toString()).get(i);
                                temp.setValue(null);
                                tempRes.put(temp.getParent(), temp);
                            }
                        }

                        startAddr = dbit.getBitAddress().toString();
                        startIndex = lastIndex + 1;
                    }
                    lasetAddr = dbit.getBitAddress().toString();
                    lastIndex += 1;
                }
                short len = (short) (Integer.parseInt(lasetAddr.substring(1)) - Integer.parseInt(startAddr.substring(1)) + 1);
                OperateResultExOne<boolean[]> res = ReadBool(startAddr, len);
                if (res.IsSuccess && res.Content.length == len) {
//                    System.out.println(res.Content);

                    for (int i = startIndex; i <= lastIndex; i++) {
                        BasicDataBit temp = Zones.get(z.toString()).get(i);
                        temp.setValue(res.Content[Integer.parseInt(temp.getBitAddress().toString().substring(1)) - Integer.parseInt(startAddr.substring(1))]);
                        tempRes.put(temp.getParent(), temp);
                    }
                } else {
                    for (int i = startIndex; i <= lastIndex; i++) {
                        BasicDataBit temp = Zones.get(z.toString()).get(i);
                        temp.setValue(null);
                        tempRes.put(temp.getParent(), temp);
                    }
                }
            }

        }

        for (Integer z = 3; z <= 4; z++) {
            if (Zones.get(z.toString()).size() > 0) {
                DataBit startBit = Zones.get(z.toString()).get(0);
                DataBit lastBit = Zones.get(z.toString()).get(0);
                int startIndex = 0;
                int lastIndex = -1;
                short typeLen = 1;
                for (DataBit dbit : Zones.get(z.toString())) {
                    if (Integer.parseInt(dbit.getBitAddress().toString().substring(1)) - Integer.parseInt(lastBit.getBitAddress().toString().substring(1)) > 200) {
                        if (lastBit.getTypeDefine() == DataType.UINT32
                                || lastBit.getTypeDefine() == DataType.INT32
                                || lastBit.getTypeDefine() == DataType.FLOAT) {
                            typeLen = 2;
                        } else if (lastBit.getTypeDefine() == DataType.INT64 || lastBit.getTypeDefine() == DataType.DOUBLE) {
                            typeLen = 4;
                        }
                        short len = (short) (Integer.parseInt(lastBit.getBitAddress().toString().substring(1)) - Integer.parseInt(startBit.getBitAddress().toString().substring(1)) + typeLen);
                        OperateResultExOne<byte[]> res = Read(startBit.getBitAddress().toString(), len);
                        if (res.IsSuccess && res.Content.length == len * 2) {
                            for (int i = startIndex; i <= lastIndex; i++) {
                                BasicDataBit temp = Zones.get(z.toString()).get(i);

                                int startIndex1 = Integer.parseInt(temp.getBitAddress().toString().substring(1)) - Integer.parseInt(startBit.getBitAddress().toString().substring(1));


                                Object value = dataTypeConvert(res.Content, startIndex1 * 2, temp);
                                temp.setValue(value);
                                tempRes.put(temp.getParent(), temp);
                            }


                        } else {
                            for (int i = startIndex; i <= lastIndex; i++) {
                                BasicDataBit temp = Zones.get(z.toString()).get(i);
                                temp.setValue(null);
                                tempRes.put(temp.getParent(), temp);
                            }
                        }

                        startBit = dbit;
                        startIndex = lastIndex + 1;

                    }
                    lastBit = dbit;
                    lastIndex += 1;
                }

                typeLen = 1;
                if (lastBit.getTypeDefine() == DataType.UINT32
                        || lastBit.getTypeDefine() == DataType.INT32
                        || lastBit.getTypeDefine() == DataType.FLOAT) {
                    typeLen = 2;
                } else if (lastBit.getTypeDefine() == DataType.INT64 || lastBit.getTypeDefine() == DataType.DOUBLE) {
                    typeLen = 4;
                }


                short len = (short) (Integer.parseInt(lastBit.getBitAddress().toString().substring(1)) - Integer.parseInt(startBit.getBitAddress().toString().substring(1)) + typeLen);

                OperateResultExOne<byte[]> res = Read(startBit.getBitAddress().toString(), len);
                if (res.IsSuccess && res.Content.length == len * 2) {

                    for (int i = startIndex; i <= lastIndex; i++) {
                        BasicDataBit temp = Zones.get(z.toString()).get(i);
                        int startIndex1 = Integer.parseInt(temp.getBitAddress().toString().substring(1)) - Integer.parseInt(startBit.getBitAddress().toString().substring(1));
                        Object value = dataTypeConvert(res.Content, startIndex1 * 2, temp);
                        temp.setValue(value);

                        tempRes.put(temp.getParent(), temp);
                    }
                } else {
                    for (int i = startIndex; i <= lastIndex; i++) {
                        BasicDataBit temp = Zones.get(z.toString()).get(i);
                        temp.setValue(null);
                        tempRes.put(temp.getParent(), temp);
                    }
                }
            }

        }


        for (BasicDataBit dbit : Zones.get("array")) {
            float typeLen = modbusValueType.byteLength(dbit.getTypeDefine());
            OperateResultExOne<byte[]> res = Read(dbit.getBitAddress().toString(), (short) (dbit.getDataNumbers() * typeLen));
            if (res.IsSuccess && res.Content.length == (short) (dbit.getDataNumbers() * typeLen * 2)) {
                Object value = dataTypeConvert(res.Content, 0, dbit);
                dbit.setValue(value);
                tempRes.put(dbit.getParent(), dbit);
            } else {
                dbit.setValue(null);
                tempRes.put(dbit.getParent(), dbit);
            }
        }


        DataBlock DBRes = new BasicDataBlock(tempRes);

        return DBRes;
    }

    private Object dataTypeConvert(byte[] inData, int startIndex, DataType type) {
        switch (type) {
            case UINT16:
                return byteTransform.TransUInt16(inData, startIndex);
            case INT16:
                return byteTransform.TransInt16(inData, startIndex);
            case UINT32:
                return byteTransform.TransUInt32(inData, startIndex);
            case INT32:
                return byteTransform.TransInt32(inData, startIndex);
            case INT64:
                return byteTransform.TransInt64(inData, startIndex);
            case FLOAT:
                return byteTransform.TransSingle(inData, startIndex);
            case DOUBLE:
                return byteTransform.TransDouble(inData, startIndex);

            case BCD:
                short bcdsi = 0;
                byte[] bcdsRes = new byte[inData.length * 2];
                for (; bcdsi < inData.length; ) {
                    bcdsRes[bcdsi * 2 + 1] = (byte) (inData[startIndex + bcdsi] & 0x0F);
                    bcdsRes[bcdsi * 2] = (byte) ((inData[startIndex + bcdsi] >> 4) & 0x0F);
                    bcdsi = (short) (bcdsi + 1);
                }
                return bcdsRes;
            default:
                return new Object();//为什么不反回空？
        }
    }

    private Object dataTypeConvert(byte[] inData, int startIndex, BasicDataBit dataBit) {

        if (dataBit.isArray()) {
            int len = (int) modbusValueType.byteLength(dataBit.getTypeDefine());
            Object[] res = new Object[(int) (inData.length / len)];
            for (int i = 0; i < res.length; i++) {
                res[i] = dataTypeConvert(inData, startIndex + i * len, dataBit.getTypeDefine());
            }
            return res;
        }
        return dataTypeConvert(inData, startIndex, dataBit.getTypeDefine());

    }


    @Override
    public DataBlock read(DataBit dataBit) {
        return read(new DataBit[]{dataBit});
    }

    @Override
    public boolean write(DataBit parent, Object arg) {
        BasicDataBit dataBit = new BasicDataBit(parent, modbusValueType);
        if (dataBit.getBitAddress().toString().substring(0, 1) == "0") {
            OperateResultExOne<byte[]> command = BuildWriteOneCoilCommand(dataBit.getBitAddress().toString().substring(1), Boolean.parseBoolean(arg.toString()));
            if (!command.IsSuccess) {
                throw new WriteException();
//                return false;
            }
            OperateResultExOne<byte[]> res = CheckModbusTcpResponse(command.Content);
            if (!res.IsSuccess) {

                throw new WriteException();
            }
            return true;
        } else {
            OperateResultExOne<byte[]> command = BuildWriteRegisterCommand(dataBit.getBitAddress().toString().substring(1), ObjectToBytes(arg, dataBit));
            if (!command.IsSuccess) {
                throw new WriteException();
//                return false;
            }
            OperateResultExOne<byte[]> res = CheckModbusTcpResponse(command.Content);
            if (!res.IsSuccess) {
                System.out.println(Arrays.toString(res.Content));
                throw new WriteException();
//                return false;
            }
            return true;
        }

    }

    private byte[] ObjectToBytes(Object inData, DataType type) {
        switch (type) {
//            case BYTES:
//                value = (byte[]) (inData);
//                break;
            case UINT16: {
                byte[] value = byteTransform.TransByte(Integer.parseInt(inData.toString()));
                return new byte[]{value[2], value[3]};
            }
            case INT16:
                return byteTransform.TransByte(Short.parseShort(inData.toString()));
            case UINT32: {
                byte[] value = byteTransform.TransByte(Long.parseLong(inData.toString()));
                return new byte[]{value[4], value[5], value[6], value[7]};
            }
            case INT32:
                return byteTransform.TransByte(Integer.parseInt(inData.toString()));
            case INT64:
                return byteTransform.TransByte(Long.parseLong(inData.toString()));
            case FLOAT:
                return byteTransform.TransByte(Float.parseFloat(inData.toString()));
            case DOUBLE:
                return byteTransform.TransByte(Double.parseDouble(inData.toString()));
            case BCD:
                byte[] BCDSData = (byte[]) inData;
                byte[] BCDSBytes = new byte[BCDSData.length / 2];
                for (int i = 0; i < BCDSData.length; ) {
                    BCDSBytes[i / 2] = (byte) (((BCDSData[i] << 4) & 0xF0) | BCDSData[i + 1] & 0x0F);
                    i = i + 2;
                }
                return BCDSBytes;

//            case UINT16S:
//                int[] uint16SData = (int[]) inData;
//                byte[] uint16SBytes = new byte[uint16SData.length * 2];
//                for (int i = 0; i < uint16SData.length; i++) {
//                    byte[] temp = byteTransform.TransByte(uint16SData[i]);
//                    System.arraycopy(temp, 2, uint16SBytes, 2 * i, 2);
//                }
//                value = uint16SBytes;
//                break;
//            case INT16S:
//                short[] int16SData = (short[]) inData;
//                byte[] int16SBytes = new byte[int16SData.length * 2];
//                for (int i = 0; i < int16SData.length; i++) {
//                    byte[] temp = byteTransform.TransByte(int16SData[i]);
//                    System.arraycopy(temp, 0, int16SBytes, 2 * i, 2);
//                }
//                value = int16SBytes;
//                break;
//            case UINT32S:
//                long[] uint32SData = (long[]) inData;
//                byte[] uint32SBytes = new byte[uint32SData.length * 4];
//                for (int i = 0; i < uint32SData.length; i++) {
//                    byte[] temp = byteTransform.TransByte(uint32SData[i]);
//                    System.arraycopy(temp, 4, uint32SBytes, 4 * i, 4);
//                }
//                value = uint32SBytes;
//                break;
//            case INT32S:
//                int[] int32SData = (int[]) inData;
//                byte[] int32SBytes = new byte[int32SData.length * 4];
//                for (int i = 0; i < int32SData.length; i++) {
//                    byte[] temp = byteTransform.TransByte(int32SData[i]);
//                    System.arraycopy(temp, 0, int32SBytes, 4 * i, 4);
//                }
//                value = int32SBytes;
//                break;
//            case INT64S:
//                long[] int64SData = (long[]) inData;
//                byte[] int64SBytes = new byte[int64SData.length * 8];
//                for (int i = 0; i < int64SData.length; i++) {
//                    byte[] temp = byteTransform.TransByte(int64SData[i]);
//                    System.arraycopy(temp, 0, int64SBytes, 8 * i, 8);
//                }
//                value = int64SBytes;
//                break;
//            case FLOATS:
//                float[] floatSData = (float[]) inData;
//                byte[] floatSBytes = new byte[floatSData.length * 4];
//                for (int i = 0; i < floatSData.length; i++) {
//                    byte[] temp = byteTransform.TransByte(floatSData[i]);
//                    System.arraycopy(temp, 0, floatSBytes, 4 * i, 4);
//                }
//                value = floatSBytes;
//                break;
//            case DOUBLES:
//                double[] doubleSData = (double[]) inData;
//                byte[] doubleSBytes = new byte[doubleSData.length * 8];
//                for (int i = 0; i < doubleSData.length; i++) {
//                    byte[] temp = byteTransform.TransByte(doubleSData[i]);
//                    System.arraycopy(temp, 0, doubleSBytes, 8 * i, 8);
//                }
//                value = doubleSBytes;
//                break;
            default:
                return new byte[0];
        }
    }

    private byte[] ObjectToBytes(Object inData, BasicDataBit type) {
        if (type.isArray()) {
            Object[] in = (Object[]) inData;
            int len = (int) (modbusValueType.byteLength(type.getTypeDefine()) * 2);
            byte[] val = new byte[(int) (in.length * len)];
            for (int i = 0; i < in.length; i++) {
                byte[] u = ObjectToBytes(in[i], type.getTypeDefine());
                System.arraycopy(u, 0, val, len * i, len);
            }
            return val;
        }

        return ObjectToBytes(inData, type.getTypeDefine());

    }

    @Override
    public boolean writeBatch(DataBit[] dataBits, Object[] args) {
        if (dataBits.length != args.length) return false;
        boolean res = true;
        for (int i = 0; i < dataBits.length; i++) {
            if (!write(dataBits[i], args[i])) {
                res = false;
            }
        }
        return res;
    }

    @Override
    public void asyncWriteBatch(DataBit[] dataBits, Object[] args, Consumer<WriteCompleted> consumer) {
        CompletableFuture<Void> complete = CompletableFuture.runAsync(() -> {
            writeBatch(dataBits, args);
        }).thenAccept(s -> {
            consumer.accept(new WriteCompleted("success", LocalDateTime.now(), null));
        });


    }


    /**********************************************************************************************/


    /*************************************以下为协议部分********************************************************/
    public OperateResultExOne<byte[]> Read(String address, short length) {


//        Integer startAddr=Integer.parseInt(address.substring(1, -1));
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
//
        int alreadyFinished = 0;
        while (alreadyFinished < length) {

            short lengthTmp = (short) Math.min((length - alreadyFinished), this.pageSize);
//            System.out.println(String.valueOf((Integer.parseInt(address.substring(1))+alreadyFinished)));
            OperateResultExOne<byte[]> read = ReadModBusBase(address.substring(0, 1) + String.valueOf((Integer.parseInt(address.substring(1)) + alreadyFinished)), lengthTmp);
            if (!read.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(read);

            try {
                outputStream.write(read.Content);
            } catch (Exception ex) {

            }
            alreadyFinished += lengthTmp;
        }

        byte[] data = outputStream.toByteArray();
        try {
            outputStream.close();
        } catch (Exception ex) {

        }
        return OperateResultExOne.CreateSuccessResult(data);
    }


    public OperateResultExOne<byte[]> Read(String address) {

        return Read(address, (short) 1);


    }

    /**
     * 获取起始地址是否从0开始
     *
     * @return bool值
     */
    public boolean getAddressStartWithZero() {
        return isAddressStartWithZero;
    }

    /**
     * 设置起始地址是否从0开始
     *
     * @param addressStartWithZero true代表从0开始，false代表从1开始
     */
    public void setAddressStartWithZero(boolean addressStartWithZero) {
        this.isAddressStartWithZero = addressStartWithZero;
    }

    /**
     * 获取站号
     *
     * @return 站号
     */
    public byte getStation() {
        return station;
    }

    /**
     * 设置站号
     *
     * @param station 站号
     */
    public void setStation(byte station) {
        this.station = station;
    }

    /**
     * 设置多字节数据的反转类型，适用于int,float,double,long类型的数据
     *
     * @param dataFormat 数据类型
     */
    public void setDataFormat(DataFormat dataFormat) {
        getByteTransform().setDataFormat(dataFormat);
    }

    /**
     * 获取多字节数据的反转类型，适用于int,float,double,long类型的数据
     *
     * @return
     */
    public DataFormat getDataFormat() {
        return getByteTransform().getDataFormat();
    }


    /**
     * 字符串数据是否发生反转
     *
     * @return bool值
     */
    public boolean isStringReverse() {
        return getByteTransform().IsStringReverse;
    }

    /**
     * 设置字符串数据是否反转
     *
     * @param stringReverse bool值
     */
    public void setStringReverse(boolean stringReverse) {
        getByteTransform().IsStringReverse = stringReverse;
    }


    private OperateResultExOne<byte[]> BuildReadCoilCommand(String address, short count) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateReadCoils(station, count), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }

    private OperateResultExOne<byte[]> BuildReadDiscreteCommand(String address, short length) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateReadDiscrete(station, length), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }


    private OperateResultExOne<byte[]> BuildReadRegisterCommand(String address, short length) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateReadRegister(station, length), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }

    private OperateResultExOne<byte[]> BuildReadInputRegisterCommand(String address, short length) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateReadInputRegister(station, length), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }

    private OperateResultExOne<byte[]> BuildReadRegisterCommand(ModbusAddress address, short length) {
        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(address.CreateReadRegister(station, length), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }


    private OperateResultExOne<byte[]> BuildWriteOneCoilCommand(String address, boolean value) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateWriteOneCoil(station, value), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }


    private OperateResultExOne<byte[]> BuildWriteOneRegisterCommand(String address, byte[] data) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateWriteOneRegister(station, data), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }


    private OperateResultExOne<byte[]> BuildWriteCoilCommand(String address, boolean[] values) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateWriteCoil(station, values), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }

    private OperateResultExOne<byte[]> BuildWriteRegisterCommand(String address, byte[] values) {
        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        short messageId = (short) softIncrementCount.GetCurrentValue();
        // 生成最终tcp指令
        byte[] buffer = ModbusInfo.PackCommandToTcp(analysis.Content.CreateWriteRegister(station, values), messageId);
        return OperateResultExOne.CreateSuccessResult(buffer);
    }


    /**
     * 检查当前的Modbus-Tcp响应是否是正确的
     *
     * @param send 发送的数据信息
     * @return 带是否成功的结果数据
     */
    private OperateResultExOne<byte[]> CheckModbusTcpResponse(byte[] send) {
        OperateResultExOne<byte[]> result = ReadFromCoreServer(send);

        if (result.IsSuccess) {
            if ((send[7] + 0x80) == result.Content[7]) {
                // 发生了错误
                result.IsSuccess = false;
                result.Message = ModbusInfo.GetDescriptionByErrorCode(result.Content[8]);
                result.ErrorCode = result.Content[8];
            }
        }
        return result;
    }

    /**
     * 读取服务器的数据，需要指定不同的功能码
     *
     * @param address 地址
     * @param length  长度
     * @return 结果数据
     */
//    private OperateResultExOne<byte[]> ReadModBusBase(ModbusAddress address, short length) {
//        OperateResultExOne<byte[]> command = BuildReadRegisterCommand(address, length);
//        if (!command.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(command);
//
//        OperateResultExOne<byte[]> resultBytes = CheckModbusTcpResponse(command.Content);
//        if (resultBytes.IsSuccess) {
//            // 二次数据处理
//            if (resultBytes.Content.length >= 9) {
//                byte[] buffer = new byte[resultBytes.Content.length - 9];
//                System.arraycopy(resultBytes.Content, 9, buffer, 0, buffer.length);
//                resultBytes.Content = buffer;
//            }
//        }
//        return resultBytes;
//    }


    /**
     * 读取服务器的数据，需要指定不同的功能码
     *
     * @param address 地址
     * @param length  长度
     * @return 指令字节信息
     */
    private OperateResultExOne<byte[]> ReadModBusBase(String address, short length) {
        byte code = Byte.parseByte(address.substring(0, 1));
        OperateResultExOne<byte[]> command = null;
        switch (code) {
            case 0: {
                command = BuildReadCoilCommand(address.substring(1), length);
                break;
            }
            case 1: {
                command = BuildReadDiscreteCommand(address.substring(1), length);
                break;
            }
            case 3: {

                command = BuildReadInputRegisterCommand(address.substring(1), length);
                break;
            }
            case 4: {
                command = BuildReadRegisterCommand(address.substring(1), length);
                break;
            }
            default: {
                command = new OperateResultExOne<byte[]>();
                command.Message = StringResources.Language.ModbusTcpFunctionCodeNotSupport();
                break;
            }
        }
        if (!command.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(command);

        OperateResultExOne<byte[]> resultBytes = CheckModbusTcpResponse(command.Content);
        if (resultBytes.IsSuccess) {
            // 二次数据处理
            if (resultBytes.Content.length >= 9) {
                byte[] buffer = new byte[resultBytes.Content.length - 9];
                System.arraycopy(resultBytes.Content, 9, buffer, 0, buffer.length);
                resultBytes.Content = buffer;
            }
        }
        return resultBytes;

    }


    /**
     * 读取线圈，需要指定起始地址
     *
     * @param address 起始地址，格式为"1234"
     * @return 带有成功标志的bool对象
     */
    public OperateResultExOne<Boolean> ReadBool(String address) {
        OperateResultExOne<byte[]> read = ReadModBusBase(address, (short) 1);
        if (!read.IsSuccess) return OperateResultExOne.<Boolean>CreateFailedResult(read);

        return GetBoolResultFromBytes(read);
    }


    /**
     * 批量的读取线圈，需要指定起始地址，读取长度
     *
     * @param address 起始地址，格式为"1234"
     * @param length  读取长度
     * @return 带有成功标志的bool数组对象
     */
    public OperateResultExOne<boolean[]> ReadBool(String address, short length) {

//        OperateResultExOne<byte[]> read = ReadModBusBase(address, length);
//        if (!read.IsSuccess) return OperateResultExOne.<boolean[]>CreateFailedResult(read);
//
//        return OperateResultExOne.CreateSuccessResult(SoftBasic.ByteToBoolArray(read.Content, length));

        OperateResultExOne<ModbusAddress> analysis = ModbusInfo.AnalysisReadAddress(address, isAddressStartWithZero);
        if (!analysis.IsSuccess) return OperateResultExOne.<boolean[]>CreateFailedResult(analysis);

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
//
        int alreadyFinished = 0;
        while (alreadyFinished < length) {

            short lengthTmp = (short) Math.min((length - alreadyFinished), this.pageSize);
//            System.out.println(lengthTmp);
            OperateResultExOne<byte[]> read = ReadModBusBase(address.substring(0, 1) + String.valueOf((Integer.parseInt(address.substring(1)) + alreadyFinished)), lengthTmp);
            if (!read.IsSuccess) return OperateResultExOne.<boolean[]>CreateFailedResult(read);

            try {
                outputStream.write(read.Content);
            } catch (Exception ex) {

            }
            alreadyFinished += lengthTmp;
        }

        byte[] data = outputStream.toByteArray();
        try {
            outputStream.close();
        } catch (Exception ex) {

        }
        return OperateResultExOne.CreateSuccessResult(SoftBasic.ByteToBoolArray(data, length));
//        return OperateResultExOne.CreateSuccessResult(data);


    }


//    /**
//     * 读取输入线圈，需要指定起始地址
//     *
//     * @param address 起始地址，格式为"1234"
//     * @return 带有成功标志的bool对象
//     */
//    public OperateResultExOne<Boolean> ReadDiscrete(String address) {
//        OperateResultExOne<byte[]> read = ReadModBusBase(address, (short) 1);
//        if (!read.IsSuccess) return OperateResultExOne.<Boolean>CreateFailedResult(read);
//
//        return GetBoolResultFromBytes(read);
//    }
//
//
//    /**
//     * 批量的读取输入点，需要指定起始地址，读取长度
//     *
//     * @param address 起始地址，格式为"1234"
//     * @param length  读取长度
//     * @return 带有成功标志的bool数组对象
//     */
//    public OperateResultExOne<boolean[]> ReadDiscrete(String address, short length) {
//        OperateResultExOne<byte[]> read = ReadModBusBase( address, length);
//        if (!read.IsSuccess) return OperateResultExOne.<boolean[]>CreateFailedResult(read);
//
//        return OperateResultExOne.CreateSuccessResult(SoftBasic.ByteToBoolArray(read.Content, length));
//    }


    /**
     * 写一个寄存器数据
     *
     * @param address 起始地址
     * @param high    高位
     * @param low     地位
     * @return 返回写入结果
     */
    public OperateResult WriteOneRegister(String address, byte high, byte low) {
        OperateResultExOne<byte[]> command = BuildWriteOneRegisterCommand(address, new byte[]{high, low});
        if (!command.IsSuccess) {
            return command;
        }

        return CheckModbusTcpResponse(command.Content);
    }

    /**
     * 写一个寄存器数据
     *
     * @param address 起始地址
     * @param value   写入值
     * @return 返回写入结果
     */
    public OperateResult WriteOneRegister(String address, short value) {
        byte[] buffer = Utilities.getBytes(value);
        return WriteOneRegister(address, buffer[1], buffer[0]);
    }


}
