package com.gct.driver.siemens;

import com.gct.driver.core.*;

import com.gct.driver.core.net.NetworkDeviceBase;
import com.gct.driver.lotpi.*;
import com.gct.driver.lotpi.exception.ConnectException;
import com.gct.driver.lotpi.exception.NotImplemented;
import com.gct.driver.lotpi.exception.UrlInvalid;
import com.gct.driver.lotpi.exception.WriteException;
import com.gct.driver.lotpi.extent.Connector;
import com.gct.driver.lotpi.message.WriteCompleted;
import com.gct.driver.core.transfer.ReverseBytesTransform;
import com.gct.driver.core.operate.OperateResult;
import com.gct.driver.core.operate.OperateResultExOne;
import com.gct.driver.core.operate.OperateResultExThree;
import com.gct.driver.lotpi.util.DriverUrlParser;
import sun.util.calendar.LocalGregorianCalendar;

import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.ObjectOutputStream;
import java.io.UnsupportedEncodingException;
import java.net.Socket;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.HashMap;
import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.function.Consumer;

/**
 * 西门子的数据交互类，采用s7协议实现
 */
public class SiemensS7 extends NetworkDeviceBase<S7Message, ReverseBytesTransform> implements Connector {

    /**
     * 实例化一个西门子的S7协议的通讯对象并指定Ip地址
     */
    public SiemensS7() {
        super(S7Message.class, ReverseBytesTransform.class);

    }

    private SiemensValueType siemensValueType = new SiemensValueType();
    private int pageSize;

    /****************************************************************************/
    /**
     * 在客户端连接上服务器后，所做的一些初始化操作 ->
     * Two handshake actions required after connecting to the server
     *
     * @param socket 网络套接字
     * @return 返回连接结果
     */
    @Override
    protected OperateResult InitializationOnConnect(Socket socket) {
        // 第一层通信的初始化
        OperateResultExOne<byte[]> read_first = ReadFromCoreServer(socket, plcHead1);
        if (!read_first.IsSuccess) return read_first;

        // 第二层通信的初始化
        OperateResultExOne<byte[]> read_second = ReadFromCoreServer(socket, plcHead2);
        if (!read_second.IsSuccess) return read_second;

        // 返回成功的信号
        return OperateResult.CreateSuccessResult();
    }


    @Override
    public void setConfig(ConnectionConfig config) {
        if (!(config instanceof SiemensConfig)) throw new NotImplemented("先使用parseUrl转换");
        SiemensConfig actualConfig = (SiemensConfig) config;
        this.CurrentPlc = actualConfig.getDriverType();
        this.setIpAddress(actualConfig.getIp());
        this.setPort(actualConfig.getPort());
        this.setTimeout(actualConfig.getTimeout());
        this.setRack(Byte.parseByte(actualConfig.getRack()));
        this.setSlot(Byte.parseByte(actualConfig.getSlot()));
        this.pageSize = actualConfig.getPageSize();
        WordLength = 2;

    }

    @Override
    public ConnectionConfig parseUrl(String url) {

        try {
            DriverUrlParser uri = DriverUrlParser.tryParse(url);
            SiemensConfig siemensConfig = new SiemensConfig();
            siemensConfig.setIp(uri.getHost());
            siemensConfig.setPort(uri.getPort());
            siemensConfig.setRack(uri.getParamByKey("rack"));
            siemensConfig.setSlot(uri.getParamByKey("slot"));
            siemensConfig.setByteOrder(Integer.parseInt(uri.getParamByKey("byteOrder")));
            String pageSize = uri.getParamByKey("pageSize");
            if (Objects.nonNull(pageSize))
                siemensConfig.setPageSize(Integer.parseInt(pageSize));
            return siemensConfig;
        } catch (Exception e) {
            throw new UrlInvalid("Unhandled exception: " + e.getMessage());
        }

    }

    @Override
    public boolean open() throws ConnectException {
        try {
            OperateResult res = this.ConnectServer();
            if (res.IsSuccess) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            throw new ConnectException("connect to " + getIpAddress() + ":" + getPort() + " failed");
        }

    }

    @Override
    public boolean close() throws ConnectException {
        try {
            OperateResult res = this.ConnectClose();
            if (res.IsSuccess) {
                return true;
            } else {
                return false;
            }
        } catch (Exception e) {
            throw new ConnectException("close " + getIpAddress() + ":" + getPort() + " failed");
        }

    }

    @Override
    public boolean isOpen() {
        return !IsSocketError;
    }

    @Override
    public void setTimeout(int timeout) {
        this.setConnectTimeOut(timeout);
    }

    @Override
    public boolean preReading(DataBit[] dataBits) {
        return false;
    }

    @Override
    public DataBlock read(DataBit[] parent) {
        BasicDataBit[] dataBits = BasicDataBit.transFromArray(parent, siemensValueType);
        HashMap<String, ArrayList<BasicDataBit>> ZonesBool = new HashMap<>();
        HashMap<String, ArrayList<Integer>> ZonesBoolSlice = new HashMap<>();
        HashMap<String, ArrayList<BasicDataBit>> ZonesAnl = new HashMap<>();
        HashMap<String, ArrayList<Integer>> ZonesAnlSlice = new HashMap<>();


        HashMap<DataBit, BasicDataBit> tempRes = new HashMap<>();

        //分类
        for (BasicDataBit dBit : dataBits) {
            if (dBit.getTypeDefine() == DataType.BOOL) {
                if (dBit.isArray()) {
                    if (ZonesBool.get("array") == null) {
                        ZonesBool.put("array", new ArrayList<BasicDataBit>());
                    }
                    ZonesBool.get("array").add(dBit);
                } else if (dBit.getBitAddress().toString().substring(0, 2).equals("DB")) {
                    if (ZonesBool.get(dBit.getBitAddress().toString().split("\\.")[0]) == null) {
                        ZonesBool.put(dBit.getBitAddress().toString().split("\\.")[0], new ArrayList<BasicDataBit>());
                    }
                    ZonesBool.get(dBit.getBitAddress().toString().split("\\.")[0]).add(dBit);
                } else {
                    if (ZonesBool.get(dBit.getBitAddress().toString().substring(0, 1)) == null) {
                        ZonesBool.put(dBit.getBitAddress().toString().substring(0, 1), new ArrayList<BasicDataBit>());
                    }

                    ZonesBool.get(dBit.getBitAddress().toString().substring(0, 1)).add(dBit);
                }
            } else {
                if (dBit.isArray()) {
                    if (ZonesAnl.get("array") == null) {
                        ZonesAnl.put("array", new ArrayList<BasicDataBit>());
                    }
                    ZonesAnl.get("array").add(dBit);
                } else if (dBit.getBitAddress().toString().substring(0, 2).equals("DB")) {
                    if (ZonesAnl.get(dBit.getBitAddress().toString().split("\\.")[0]) == null) {
                        ZonesAnl.put(dBit.getBitAddress().toString().split("\\.")[0], new ArrayList<BasicDataBit>());
                    }
                    ZonesAnl.get(dBit.getBitAddress().toString().split("\\.")[0]).add(dBit);
                } else {
                    if (ZonesAnl.get(dBit.getBitAddress().toString().substring(0, 1)) == null) {
                        ZonesAnl.put(dBit.getBitAddress().toString().substring(0, 1), new ArrayList<BasicDataBit>());
                    }
                    ZonesAnl.get(dBit.getBitAddress().toString().substring(0, 1)).add(dBit);
                }
            }
        }


//        System.out.println(Arrays.toString(ZonesBool.get("Q").get(0).getBitAddress().toString().split("\\.")));
        //排序
        for (String key : ZonesBool.keySet()) {
            if (key == "array") {
                continue;
            }
            if (ZonesBool.get(key).size() > 1) {
                if (key.length() >= 3) {
                    ZonesBool.get(key).sort((DataBit b1, DataBit b2) -> (int) (
                            Integer.parseInt(b1.getBitAddress().toString().split("\\.")[1]) - Integer.parseInt(b2.getBitAddress().toString().split("\\.")[1])
                    ));
                } else {
                    ZonesBool.get(key).sort((DataBit b1, DataBit b2) -> (int) (

                            Integer.parseInt(b1.getBitAddress().toString().split("\\.")[0].substring(1)) - Integer.parseInt(b2.getBitAddress().toString().split("\\.")[0].substring(1))
                    ));
                }
            }

        }

        for (String key : ZonesAnl.keySet()) {
            if (key == "array") {
                continue;
            }
            if (ZonesAnl.get(key).size() > 1) {
                if (key.length() >= 3) {
                    ZonesAnl.get(key).sort((DataBit b1, DataBit b2) -> (int) (
                            Integer.parseInt(b1.getBitAddress().toString().split("\\.")[1]) - Integer.parseInt(b2.getBitAddress().toString().split("\\.")[1])
                    ));
                } else {
                    ZonesAnl.get(key).sort((DataBit b1, DataBit b2) -> (int) (
                            Integer.parseInt(b1.getBitAddress().toString().split("\\.")[0].substring(1)) - Integer.parseInt(b2.getBitAddress().toString().split("\\.")[0].substring(1))
                    ));
                }
            }

        }


        //切分
        for (String key : ZonesBool.keySet()) {
            if (key == "array") {
                continue;
            }
            int index = 0;
            ZonesBoolSlice.put(key, new ArrayList<>());
            if (key.length() >= 3) {
                String lasetAddr = ZonesBool.get(key).get(0).getBitAddress().toString();
                for (DataBit dbit : ZonesBool.get(key)) {
                    if (Integer.parseInt(dbit.getBitAddress().toString().split("\\.")[1]) - Integer.parseInt(lasetAddr.split("\\.")[1]) > 2000) {
                        ZonesBoolSlice.get(key).add(index - 1);
                    }
                    index++;
                    lasetAddr = dbit.getBitAddress().toString();
                }
                ZonesBoolSlice.get(key).add(index - 1);

            } else {
                String lasetAddr = ZonesBool.get(key).get(0).getBitAddress().toString();
                for (DataBit dbit : ZonesBool.get(key)) {
                    if (Integer.parseInt(dbit.getBitAddress().toString().split("\\.")[0].substring(1)) - Integer.parseInt(lasetAddr.split("\\.")[0].substring(1)) > 2000) {
                        ZonesBoolSlice.get(key).add(index - 1);
                    }
                    index++;
                    lasetAddr = dbit.getBitAddress().toString();
                }
                ZonesBoolSlice.get(key).add(index - 1);
            }
        }
        for (String key : ZonesAnl.keySet()) {
            if (key == "array") {
                continue;
            }
            int index = 0;
            ZonesAnlSlice.put(key, new ArrayList<>());
            if (key.length() >= 3) {
                String lasetAddr = ZonesAnl.get(key).get(0).getBitAddress().toString();
                for (DataBit dbit : ZonesAnl.get(key)) {
                    if (Integer.parseInt(dbit.getBitAddress().toString().split("\\.")[1]) - Integer.parseInt(lasetAddr.split("\\.")[1]) > 2000) {
                        ZonesAnlSlice.get(key).add(index - 1);
                    }
                    index++;
                    lasetAddr = dbit.getBitAddress().toString();
                }
                ZonesAnlSlice.get(key).add(index - 1);

            } else {
                String lasetAddr = ZonesAnl.get(key).get(0).getBitAddress().toString();
                for (DataBit dbit : ZonesAnl.get(key)) {
                    if (Integer.parseInt(dbit.getBitAddress().toString().split("\\.")[0].substring(1)) - Integer.parseInt(lasetAddr.split("\\.")[0].substring(1)) > 2000) {
                        ZonesAnlSlice.get(key).add(index - 1);
                    }
                    index++;
                    lasetAddr = dbit.getBitAddress().toString();
                }
                ZonesAnlSlice.get(key).add(index - 1);
            }
        }

        //读取
        for (String key : ZonesBool.keySet()) {
            int lastIndex = -1;
            if (key == "array") {

                continue;

            } else if (key.length() >= 3) {
                for (int index : ZonesBoolSlice.get(key)) {
                    short len = (short) (Integer.parseInt(ZonesBool.get(key).get(index).getBitAddress().toString().split("\\.")[1])
                            - Integer.parseInt(ZonesBool.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[1]) + 1);

                    OperateResultExOne<boolean[]> res = ReadBool(ZonesBool.get(key).get(lastIndex + 1).getBitAddress().toString(), (short) (len * 8));
                    if (res.IsSuccess && res.Content.length == len * 8) {
                        for (int i = lastIndex + 1; i <= index; i++) {
                            BasicDataBit temp = ZonesBool.get(key).get(i);
                            int byteIndex = Integer.parseInt(temp.getBitAddress().toString().split("\\.")[1])
                                    - Integer.parseInt(ZonesBool.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[1]);
                            temp.setValue(res.Content[byteIndex * 8 + Integer.parseInt(temp.getBitAddress().toString().split("\\.")[2])]);
                            tempRes.put(temp.getParent(), temp);
                        }

                    } else {
                        for (int i = 0; i < ZonesBool.get(key).size(); i++) {
                            BasicDataBit temp = ZonesBool.get(key).get(i);
                            temp.setValue(null);
                            tempRes.put(temp.getParent(), temp);
                        }
                    }
                    lastIndex = index;
                }
            } else {
                for (int index : ZonesBoolSlice.get(key)) {

                    short len = (short) (Integer.parseInt(ZonesBool.get(key).get(index).getBitAddress().toString().split("\\.")[0].substring(1)) - Integer.parseInt(ZonesBool.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[0].substring(1)) + 1);
                    OperateResultExOne<boolean[]> res = ReadBool(ZonesBool.get(key).get(lastIndex + 1).getBitAddress().toString(), (short) (len * 8));
                    if (res.IsSuccess && res.Content.length == len * 8) {
                        for (int i = lastIndex + 1; i <= index; i++) {
                            BasicDataBit temp = ZonesBool.get(key).get(i);
                            int byteIndex = Integer.parseInt(temp.getBitAddress().toString().split("\\.")[0].substring(1))
                                    - Integer.parseInt(ZonesBool.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[0].substring(1));

                            temp.setValue(res.Content[byteIndex * 8 + Integer.parseInt(temp.getBitAddress().toString().split("\\.")[1])]);
                            tempRes.put(temp.getParent(), temp);
//                            System.out.println(temp.getValue());
                        }

                    } else {
                        for (int i = 0; i < ZonesBool.get(key).size(); i++) {
                            BasicDataBit temp = ZonesBool.get(key).get(i);
                            temp.setValue(null);
                            tempRes.put(temp.getParent(), temp);
                        }
                    }
                    lastIndex = index;
                }
            }
        }
        for (String key : ZonesAnl.keySet()) {
            int lastIndex = -1;
            float typeLen = 1;
            if (key == "array") {

                for (BasicDataBit dbit : ZonesAnl.get("array")) {

                    typeLen = siemensValueType.byteLength(dbit.getTypeDefine());
                    OperateResultExOne<byte[]> res = Read(dbit.getBitAddress().toString(), (short) (dbit.getDataNumbers() * typeLen));
                    if (res.IsSuccess && res.Content.length == (short) (dbit.getDataNumbers() * typeLen)) {
                        Object value = DateTypeConvert(res.Content, 0, dbit);
                        dbit.setValue(value);
                        tempRes.put(dbit.getParent(), dbit);
                    } else {
                        dbit.setValue(null);
                        tempRes.put(dbit.getParent(), dbit);
                    }
                }

            } else if (key.length() >= 3) {
                for (int index : ZonesAnlSlice.get(key)) {
                    BasicDataBit dBit = ZonesAnl.get(key).get(index);
                    typeLen = siemensValueType.byteLength(dBit.getTypeDefine());
                    short len = (short) (Integer.parseInt(dBit.getBitAddress().toString().split("\\.")[1])
                            - Integer.parseInt(ZonesAnl.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[1]) + 1);

                    OperateResultExOne<byte[]> res = Read(ZonesAnl.get(key).get(lastIndex + 1).getBitAddress().toString(), (short) ((len + typeLen)));
                    if (res.IsSuccess && res.Content.length == (len + typeLen)) {
                        for (int i = lastIndex + 1; i <= index; i++) {
                            BasicDataBit temp = ZonesAnl.get(key).get(i);
                            int byteIndex = Integer.parseInt(temp.getBitAddress().toString().split("\\.")[1])
                                    - Integer.parseInt(ZonesAnl.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[1]);

                            temp.setValue(DateTypeConvert(res.Content, byteIndex, temp));
                            tempRes.put(temp.getParent(), temp);
                        }

                    } else {
                        for (int i = 0; i < ZonesAnl.get(key).size(); i++) {
                            BasicDataBit temp = ZonesAnl.get(key).get(i);
                            temp.setValue(null);
                            tempRes.put(temp.getParent(), temp);
                        }
                    }
                    lastIndex = index;
                }
            } else {
                for (int index : ZonesAnlSlice.get(key)) {
                    BasicDataBit dBit = ZonesAnl.get(key).get(index);
                    typeLen = siemensValueType.byteLength(dBit.getTypeDefine());
                    short len = (short) (Integer.parseInt(ZonesAnl.get(key).get(index).getBitAddress().toString().split("\\.")[0].substring(1))
                            - Integer.parseInt(ZonesAnl.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[0].substring(1)) + 1);

                    OperateResultExOne<byte[]> res = Read(ZonesAnl.get(key).get(lastIndex + 1).getBitAddress().toString(), (short) ((len + typeLen)));
                    if (res.IsSuccess && res.Content.length == (len + typeLen)) {
                        for (int i = lastIndex + 1; i <= index; i++) {
                            BasicDataBit temp = ZonesAnl.get(key).get(i);
                            int byteIndex = Integer.parseInt(temp.getBitAddress().toString().split("\\.")[0].substring(1))
                                    - Integer.parseInt(ZonesAnl.get(key).get(lastIndex + 1).getBitAddress().toString().split("\\.")[0].substring(1));

                            temp.setValue(DateTypeConvert(res.Content, byteIndex, temp));
                            tempRes.put(temp.getParent(), temp);
                        }

                    } else {
                        for (int i = 0; i < ZonesAnl.get(key).size(); i++) {
                            BasicDataBit temp = ZonesAnl.get(key).get(i);
                            temp.setValue(null);
                            tempRes.put(temp.getParent(), temp);
                        }
                    }
                    lastIndex = index;
                }
            }


        }

        DataBlock DBRes = new BasicDataBlock(tempRes);

        return DBRes;

    }

    private Object DateTypeConvert(byte[] inData, int startIndex, DataType type) {
        switch (type) {
            case BYTE:
                return byteTransform.TransByte(inData, startIndex);
            case UINT16:
                return byteTransform.TransUInt16(inData, startIndex);
            case INT16:
                return byteTransform.TransInt16(inData, startIndex);
            case UINT32:
                return byteTransform.TransUInt32(inData, startIndex);
            case INT32:
                return byteTransform.TransInt32(inData, startIndex);
            case INT64:
                return byteTransform.TransInt64(inData, startIndex);
            case FLOAT:
                return byteTransform.TransSingle(inData, startIndex);
            case DOUBLE:
                return byteTransform.TransDouble(inData, startIndex);
            case BCD:
                short bcdsi = 0;
                byte[] bcdsRes = new byte[inData.length * 2];
                for (; bcdsi < inData.length; ) {
                    bcdsRes[bcdsi * 2 + 1] = (byte) (inData[startIndex + bcdsi] & 0x0F);
                    bcdsRes[bcdsi * 2] = (byte) ((inData[startIndex + bcdsi] >> 4) & 0x0F);
                    bcdsi = (short) (bcdsi + 1);
                }
                return bcdsRes;
            default:
                return new Object();
        }
    }

    private Object DateTypeConvert(byte[] inData, int startIndex, BasicDataBit type) {
        if (type.isArray()) {
            float typeLen = siemensValueType.byteLength(type.getTypeDefine());
            int resLen = (int) (inData.length / typeLen);
            Object[] res = new Object[resLen];
            for (int i = 0; i < resLen; i++) {
                Object val = DateTypeConvert(inData, startIndex + (int) (i * typeLen), type.getTypeDefine());
                res[i] = val;
            }
            return res;
        }
        return DateTypeConvert(inData, startIndex, type.getTypeDefine());
    }

    @Override
    public DataBlock read(DataBit dataBit) {

        return read(new DataBit[]{dataBit});
    }

    @Override
    public boolean write(DataBit parent, Object arg) {
        BasicDataBit dataBit = new BasicDataBit(parent, siemensValueType);
        OperateResult res = Write(dataBit.getBitAddress().toString(), ObjectToBytes(arg, dataBit));
        if (!res.IsSuccess) {
            throw new WriteException();
        }
        return res.IsSuccess;
    }

    @Override
    public boolean writeBatch(DataBit[] dataBits, Object[] args) {
        if (dataBits.length != args.length) return false;
        boolean res = true;
        for (int i = 0; i < dataBits.length; i++) {
            if (!write(dataBits[i], args[1])) {

                throw new WriteException();
            }
        }
        return res;
    }

    @Override
    public void asyncWriteBatch(DataBit[] dataBits, Object[] args, Consumer<WriteCompleted> consumer) {
        CompletableFuture<Void> complete = CompletableFuture.runAsync(() -> {
            this.writeBatch(dataBits, args);
        }).thenAccept(s -> {

            consumer.accept(new WriteCompleted("success", LocalDateTime.now(), null));

        });
    }

    private byte[] ObjectToBytes(Object inData, DataType type) {
        switch (type) {
            case BYTE:
                return byteTransform.TransByte(Byte.parseByte(inData.toString()));
            case UINT16:
                byte[] value = byteTransform.TransByte(Integer.parseInt(inData.toString()));
                return new byte[]{value[2], value[3]};
            case INT16:
                return byteTransform.TransByte(Short.parseShort(inData.toString()));
            case UINT32:
                value = byteTransform.TransByte(Long.parseLong(inData.toString()));
                return new byte[]{value[4], value[5], value[6], value[7]};
            case INT32:
                return byteTransform.TransByte(Integer.parseInt(inData.toString()));
            case INT64:
                return byteTransform.TransByte(Long.parseLong(inData.toString()));
            case FLOAT:
                return byteTransform.TransByte(Float.parseFloat(inData.toString()));
            case DOUBLE:
                return byteTransform.TransByte(Double.parseDouble(inData.toString()));
            case BCD:
                byte[] BCDSData = (byte[]) inData;
                byte[] BCDSBytes = new byte[BCDSData.length / 2];
                for (int i = 0; i < BCDSData.length; ) {
                    BCDSBytes[i / 2] = (byte) (((BCDSData[i] << 4) & 0xF0) | BCDSData[i + 1] & 0x0F);
                    i = i + 2;
                }
                return BCDSBytes;
            default:
                return new byte[0];
        }
    }

    private byte[] ObjectToBytes(Object inData, BasicDataBit dataBit) {
        if (!dataBit.isArray()) return ObjectToBytes(inData, dataBit.getTypeDefine());

        Object[] in = (Object[]) inData;
        int typeLen = (int) siemensValueType.byteLength(dataBit.getTypeDefine());//不能支持bcd
        byte[] res = new byte[(int) (in.length * typeLen)];
        for (int i = 0; i < in.length; i++) {
            byte[] temp = ObjectToBytes(in[i], dataBit.getTypeDefine());
            System.arraycopy(temp, typeLen, res, typeLen * i, typeLen);
        }
        return res;
    }

    /**
     * 将Object对象转byte数组
     * <p>不要使用、不要使用、不要使用</p>
     *
     * @param obj byte数组的object对象
     * @return
     */
    @Deprecated
    public static byte[] toByteArray(Object obj) {
        byte[] bytes = null;
        ByteArrayOutputStream bos = new ByteArrayOutputStream();
        try {
            ObjectOutputStream oos = new ObjectOutputStream(bos);
            oos.writeObject(obj);
            oos.flush();
            bytes = bos.toByteArray();
            oos.close();
            bos.close();
        } catch (IOException ex) {
            ex.printStackTrace();
        }
        return bytes;
    }

    /******************************************************************************/

    /**
     * 从PLC读取订货号信息
     *
     * @return 订货号信息
     */
    public OperateResultExOne<String> ReadOrderNumber() {
        OperateResultExOne<String> result = new OperateResultExOne<String>();
        OperateResultExOne<byte[]> read = ReadFromCoreServer(plcOrderNumber);
        if (read.IsSuccess) {
            if (read.Content.length > 100) {
                result.IsSuccess = true;
                result.Content = Utilities.getString(Arrays.copyOfRange(read.Content, 71, 91), "ASCII");
            }
        }

        if (!result.IsSuccess) result.CopyErrorFromOther(read);

        return result;
    }


    /**
     * 从PLC读取数据，地址格式为I100，Q100，DB20.100，M100，T100，C100以字节为单位
     *
     * @param address 起始地址，格式为I100，M100，Q100，DB20.100
     * @param length  读取的数量，以字节为单位
     * @return 结果对象
     */

    public OperateResultExOne<byte[]> Read(String address, short length) {
        OperateResultExThree<Byte, Integer, Integer> addressResult = AnalysisAddress(address);
        if (!addressResult.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(addressResult);

        ByteArrayOutputStream outputStream = new ByteArrayOutputStream();

        short alreadyFinished = 0;
        while (alreadyFinished < length) {
            short readLength = (short) Math.min(length - alreadyFinished, this.pageSize);

            OperateResultExThree<Byte, Integer, Integer>[] list = new OperateResultExThree[1];
            list[0] = addressResult;

            OperateResultExOne<byte[]> read = Read(list, new short[]{readLength});
            if (read.IsSuccess) {
                try {
                    outputStream.write(read.Content);
                } catch (Exception ex) {

                }
            } else {
                return read;
            }

            alreadyFinished += readLength;
            addressResult.Content2 += readLength * 8;
        }

        byte[] buffer = outputStream.toByteArray();
//        System.out.println(Arrays.toString(buffer));

        try {
            outputStream.close();
        } catch (Exception ex) {

        }

        return OperateResultExOne.CreateSuccessResult(buffer);
    }


    public OperateResultExOne<byte[]> Read(String address) {
        return Read(address, (short) 1);

    }


    public OperateResultExOne<boolean[]> ReadBool(String address, short length) {
        OperateResultExOne<byte[]> res = Read(address, (short) (length / 8));
        OperateResultExOne<boolean[]> res2 = new OperateResultExOne<>();
        res2.IsSuccess = false;
        if (res.IsSuccess) {
            boolean[] res1 = SoftBasic.ByteToBoolArray(res.Content, length);
//            System.out.println(Arrays.toString(res1));
            res2.IsSuccess = true;
            res2.Content = res1;
        }


        return res2;
    }

    /**
     * 将数据写入到PLC数据，地址格式为I100，Q100，DB20.100，M100，以字节为单位
     *
     * @param address 起始地址，格式为I100，M100，Q100，DB20.100
     * @param value   写入的数据，长度根据data的长度来指示
     * @return 写入结果
     */
    public OperateResult Write(String address, byte[] value) {
        OperateResultExThree<Byte, Integer, Integer> analysis = AnalysisAddress(address);
        if (!analysis.IsSuccess) return OperateResultExOne.CreateFailedResult(analysis);

        int length = value.length;
        int alreadyFinished = 0;
        while (alreadyFinished < length) {
            short writeLength = (short) Math.min(length - alreadyFinished, 200);
            byte[] buffer = getByteTransform().TransByte(value, alreadyFinished, writeLength);

            OperateResultExOne<byte[]> command = BuildWriteByteCommand(analysis, buffer);
            if (!command.IsSuccess) return command;

            OperateResult write = WriteBase(command.Content);
            if (!write.IsSuccess) return write;

            alreadyFinished += writeLength;
            analysis.Content2 += writeLength * 8;
        }

        return OperateResult.CreateSuccessResult();
    }

    public OperateResult WriteBool(String address, boolean value) {
        return null;
    }

    /**
     * 初始化方法
     *
     * @param siemens   西门子类型
     * @param ipAddress Ip地址
     */
    private void Initialization(SiemensPLCS siemens, String ipAddress) {
        WordLength = 2;
        setIpAddress(ipAddress);
        setPort(102);
        CurrentPlc = siemens;

        switch (siemens) {
            case S1200:
                plcHead1[21] = 0;
                break;
            case S300:
                plcHead1[21] = 2;
                break;
            case S400:
                plcHead1[21] = 3;
                plcHead1[17] = 0;
                break;
            case S1500:
                plcHead1[21] = 0;
                break;
            case S200Smart: {
                plcHead1 = plcHead1_200smart;
                plcHead2 = plcHead2_200smart;
                break;
            }
            case S200: {
                plcHead1 = plcHead1_200;
                plcHead2 = plcHead2_200;
            }
            default:
                plcHead1[18] = 0;
                break;
        }
    }

    /**
     * 获取 PLC的槽号，针对S7-400的PLC设置的
     *
     * @return 槽号
     */
    public byte getSlot() {
        return plc_slot;
    }

    /**
     * 设置PLC的槽号信息，针对S7-400的PLC设置的
     *
     * @param value 槽号信息
     */
    public void setSlot(byte value) {
        plc_slot = value;
        plcHead1[21] = (byte) ((this.plc_rack * 0x20) + this.plc_slot);
    }

    /**
     * 获取PLC的机架号，针对S7-400的PLC设置的
     *
     * @return Plc的机架号
     */
    public byte getRack() {
        return plc_rack;
    }

    /**
     * 设置PLC的机架号，针对S7-400的PLC设置的
     *
     * @param value 机架号
     */
    public void setRack(byte value) {
        plc_rack = value;
        plcHead1[21] = (byte) ((this.plc_rack * 0x20) + this.plc_slot);
    }


    /**
     * 从PLC读取数据，地址格式为I100，Q100，DB20.100，M100，以位为单位
     *
     * @param address 起始地址，格式为I100，M100，Q100，DB20.100
     * @return 结果对象
     */
    private OperateResultExOne<byte[]> ReadBitFromPLC(String address) {
        OperateResultExOne<byte[]> command = BuildBitReadCommand(address);
        if (!command.IsSuccess) return command;

        OperateResultExOne<byte[]> read = ReadFromCoreServer(command.Content);
        if (!read.IsSuccess) return read;


        int receiveCount = 1;
        if (read.Content.length >= 21 && read.Content[20] == 1) {
            // 分析结果
            byte[] buffer = new byte[receiveCount];

            if (22 < read.Content.length) {
                if (read.Content[21] == (byte) 0xFF && read.Content[22] == 0x03) {
                    // 有数据
                    buffer[0] = read.Content[25];
                }
            }

            return OperateResultExOne.CreateSuccessResult(buffer);
        } else {
            return new OperateResultExOne<byte[]>(read.ErrorCode, StringResources.Language.SiemensDataLengthCheckFailed());
        }
    }


//    /**
//     * 一次性从PLC获取所有的数据，按照先后顺序返回一个统一的Buffer，需要按照顺序处理，两个数组长度必须一致
//     * @param address 起始地址数组
//     * @param length 数据长度数组
//     * @return 结果数据对象
//     */
//    public OperateResultExOne<byte[]> Read(String[] address, short[] length) {
//
//        OperateResultExThree<Byte, Integer, Integer>[] list = new OperateResultExThree[address.length];
//        for (int i = 0; i < address.length; i++) {
//            OperateResultExThree<Byte, Integer, Integer> tmp = AnalysisAddress(address[i]);
//            if (!tmp.IsSuccess) return OperateResultExOne.CreateFailedResult(tmp);
//
//            list[i] = tmp;
//        }
//
//        return Read(list, length);
//    }


    /**
     * 读取真实的数据
     *
     * @param address 起始地址
     * @param length  长度
     * @return 结果类对象
     */
    private OperateResultExOne<byte[]> Read(OperateResultExThree<Byte, Integer, Integer>[] address, short[] length) {

        OperateResultExOne<byte[]> command = BuildReadCommand(address, length);
        if (!command.IsSuccess) return command;

        OperateResultExOne<byte[]> read = ReadFromCoreServer(command.Content);
        if (!read.IsSuccess) return read;

        int receiveCount = 0;
        for (int i = 0; i < length.length; i++) {
            receiveCount += length[i];
        }

        if (read.Content.length >= 21 && read.Content[20] == length.length) {
            // 分析结果
            byte[] buffer = new byte[receiveCount];
            int kk = 0;
            int ll = 0;
            for (int ii = 21; ii < read.Content.length; ii++) {
                if ((ii + 1) < read.Content.length) {
                    if (read.Content[ii] == (byte) 0xFF &&
                            read.Content[ii + 1] == 0x04) {
                        // 有数据
                        System.arraycopy(read.Content, ii + 4, buffer, ll, length[kk]);
                        ii += length[kk] + 3;
                        ll += length[kk];
                        kk++;
                    }
                }
            }

            return OperateResultExOne.CreateSuccessResult(buffer);
        } else {
            return new OperateResultExOne<byte[]>(read.ErrorCode, StringResources.Language.SiemensDataLengthCheckFailed());
        }
    }


    /**
     * 读取指定地址的bool数据
     *
     * @param address 起始地址，格式为I100，M100，Q100，DB20.100
     * @return 结果类对象
     */
    public OperateResultExOne<Boolean> ReadBool(String address) {
        return GetBoolResultFromBytes(ReadBitFromPLC(address));
    }


    /**
     * 读取指定地址的byte数据
     * @param address 起始地址，格式为I100，M100，Q100，DB20.100
     * @return 结果类对象
     */
//    public OperateResultExOne<Byte> ReadByte(String address) {
//        return GetByteResultFromBytes(Read(address, (short) 1));
//    }
//


    /**
     * 基础的写入数据的操作支持
     *
     * @param entireValue 完整的字节数据
     * @return 写入结果
     */
    private OperateResult WriteBase(byte[] entireValue) {
        OperateResultExOne<byte[]> write = ReadFromCoreServer(entireValue);
        if (!write.IsSuccess) return write;

        if (write.Content[write.Content.length - 1] != (byte) 0xFF) {
            return new OperateResult(write.Content[write.Content.length - 1], StringResources.Language.SiemensWriteError() + write.Content[write.Content.length - 1]);
        } else {
            return OperateResult.CreateSuccessResult();
        }
    }


    /**
     * 写入PLC的一个位，例如"M100.6"，"I100.7"，"Q100.0"，"DB20.100.0"，如果只写了"M100"默认为"M100.0
     *
     * @param address 起始地址，格式为I100，M100，Q100，DB20.100
     * @param value   写入的数据，True或是False
     * @return 写入结果
     */
    public OperateResult Write(String address, boolean value) {
        // 生成指令
        OperateResultExOne<byte[]> command = BuildWriteBitCommand(address, value);
        if (!command.IsSuccess) return command;

        return WriteBase(command.Content);
    }


    /**
     * 向PLC中写入bool数组，返回值说明，比如你写入M100,那么data[0]对应M100.0
     *
     * @param address 要写入的数据地址
     * @param values  要写入的实际数据，长度为8的倍数
     * @return 写入结果
     */
    public OperateResult Write(String address, boolean[] values) {
        return Write(address, SoftBasic.BoolArrayToByte(values));
    }


//    /**
//     * 向PLC中写入byte数据，返回值说明
//     * @param address 要写入的数据地址
//     * @param value 要写入的实际数据
//     * @return 写入结果
//     */
//    public OperateResult Write(String address, byte value) {
//        return Write(address, new byte[]{value});
//    }


    /**
     * 读取西门子的地址的字符串信息，这个信息是和西门子绑定在一起，长度随西门子的信息动态变化的
     *
     * @param address 数据地址，具体的格式需要参照类的说明文档
     * @return 带有是否成功的字符串结果类对象
     * @throws UnsupportedEncodingException
     */
    public OperateResultExOne<String> ReadString(String address) {
        if (CurrentPlc != SiemensPLCS.S200Smart) {
            OperateResultExOne<byte[]> read = Read(address, (short) 2);
            if (!read.IsSuccess) return OperateResultExOne.<String>CreateFailedResult(read);

            if (read.Content[0] != (byte) 254) return new OperateResultExOne<String>("Value in plc is not string type");

            OperateResultExOne<byte[]> readString = Read(address, (short) (2 + read.Content[1]));
            if (!readString.IsSuccess) return OperateResultExOne.CreateFailedResult(readString);

            try {
                byte[] buffer = new byte[read.Content[1]];
                System.arraycopy(readString.Content, 2, buffer, 0, buffer.length);
                return OperateResultExOne.CreateSuccessResult(new String(buffer, "US-ASCII"));
            } catch (Exception ex) {
                return new OperateResultExOne<>(ex.getMessage());
            }
        } else {
            OperateResultExOne<byte[]> read = Read(address, (short) 1);
            if (!read.IsSuccess) return OperateResultExOne.<String>CreateFailedResult(read);

            OperateResultExOne<byte[]> readString = Read(address, (short) (1 + read.Content[0]));
            if (!readString.IsSuccess) return OperateResultExOne.CreateFailedResult(readString);

            try {
                byte[] buffer = new byte[read.Content[0]];
                System.arraycopy(readString.Content, 1, buffer, 0, buffer.length);
                return OperateResultExOne.CreateSuccessResult(new String(buffer, "US-ASCII"));
            } catch (Exception ex) {
                return new OperateResultExOne<>(ex.getMessage());
            }
        }
    }


    /**
     * 向设备中写入字符串，编码格式为ASCII
     *
     * @param address 起始地址
     * @param value 写入值
     * @return 返回读取结果
     */
//    @Override
//    public OperateResult Write(String address, String value) {
//        byte[] temp = getByteTransform().TransByte(value, "US-ASCII");
//        if (WordLength == 1) temp = SoftBasic.ArrayExpandToLengthEven(temp);
//
//        if (CurrentPlc != SiemensPLCS.S200Smart)
//        {
//            return Write( address, SoftBasic.SpliceTwoByteArray( new byte[] { (byte) 254, (byte)temp.length }, temp ) );
//        }
//        else
//        {
//            return Write( address, SoftBasic.SpliceTwoByteArray( new byte[] { (byte)temp.length }, temp ) );
//        }
//    }
//


    protected byte[] plcHead1 = new byte[]
            {
                    0x03, 0x00, 0x00, 0x16, 0x11, (byte) 0xE0, 0x00, 0x00, 0x00, 0x01, 0x00, (byte) 0xC0, 0x01,
                    0x0A, (byte) 0xC1, 0x02, 0x01, 0x02, (byte) 0xC2, 0x02, 0x01, 0x00
            };
    protected byte[] plcHead2 = new byte[]
            {
                    0x03, 0x00, 0x00, 0x19, 0x02, (byte) 0xF0, (byte) 0x80, 0x32, 0x01, 0x00, 0x00, 0x04, 0x00, 0x00, 0x08, 0x00, 0x00,
                    (byte) 0xF0, 0x00, 0x00, 0x01, 0x00, 0x01, 0x01, (byte) 0xE0
            };
    private byte[] plcOrderNumber = new byte[]
            {
                    0x03, 0x00, 0x00, 0x21, 0x02, (byte) 0xF0, (byte) 0x80, 0x32, 0x07, 0x00, 0x00, 0x00, 0x01, 0x00, 0x08, 0x00, 0x08,
                    0x00, 0x01, 0x12, 0x04, 0x11, 0x44, 0x01, 0x00, (byte) 0xFF, 0x09, 0x00, 0x04, 0x00, 0x11, 0x00, 0x00
            };
    protected SiemensPLCS CurrentPlc = SiemensPLCS.S1200;
    protected byte[] plcHead1_200smart = new byte[]
            {
                    0x03, 0x00, 0x00, 0x16, 0x11, (byte) 0xE0, 0x00, 0x00, 0x00, 0x01, 0x00, (byte) 0xC1, 0x02, 0x10, 0x00, (byte) 0xC2,
                    0x02, 0x03, 0x00, (byte) 0xC0, 0x01, 0x0A
            };
    protected byte[] plcHead2_200smart = new byte[]
            {
                    0x03, 0x00, 0x00, 0x19, 0x02, (byte) 0xF0, (byte) 0x80, 0x32, 0x01, 0x00, 0x00, (byte) 0xCC, (byte) 0xC1,
                    0x00, 0x08, 0x00, 0x00, (byte) 0xF0, 0x00, 0x00, 0x01, 0x00, 0x01, 0x03, (byte) 0xC0
            };
    protected byte[] plcHead1_200 = new byte[]
            {
                    0x03, 0x00, 0x00, 0x16, 0x11, (byte) 0xE0, 0x00, 0x00, 0x00, 0x01, 0x00, (byte) 0xC1, 0x02, 0x4D, 0x57, (byte) 0xC2,
                    0x02, 0x4D, 0x57, (byte) 0xC0, 0x01, 0x09
            };
    protected byte[] plcHead2_200 = new byte[]
            {
                    0x03, 0x00, 0x00, 0x19, 0x02, (byte) 0xF0, (byte) 0x80, 0x32, 0x01, 0x00, 0x00, 0x00, 0x00,
                    0x00, 0x08, 0x00, 0x00, (byte) 0xF0, 0x00, 0x00, 0x01, 0x00, 0x01, 0x03, (byte) 0xC0
            };

    private byte plc_slot = 0;
    private byte plc_rack = 0;

    /**
     * 返回表示当前对象的字符串
     *
     * @return 字符串信息
     */
    @Override
    public String toString() {
        return "SiemensS7Net";
    }


    /**
     * 计算特殊的地址信息
     *
     * @param address 字符串信息
     * @return 实际值
     */
    public static int CalculateAddressStarted(String address) {
        if (address.indexOf('.') < 0) {
            return Integer.parseInt(address) * 8;
        } else {
            String[] temp = address.split("\\.");
            return Integer.parseInt(temp[0]) * 8 + Integer.parseInt(temp[1]);
        }
    }


    /**
     * 解析数据地址，解析出地址类型，起始地址，DB块的地址
     *
     * @param address 数据地址
     * @return 解析出地址类型，起始地址，DB块的地址
     */
    public static OperateResultExThree<Byte, Integer, Integer> AnalysisAddress(String address) {
        OperateResultExThree<Byte, Integer, Integer> result = new OperateResultExThree<Byte, Integer, Integer>();
        try {
            result.Content3 = 0;
            if (address.charAt(0) == 'I') {
                result.Content1 = (byte) 0x81;
                result.Content2 = CalculateAddressStarted(address.substring(1));
            } else if (address.charAt(0) == 'Q') {
                result.Content1 = (byte) 0x82;
                result.Content2 = CalculateAddressStarted(address.substring(1));
            } else if (address.charAt(0) == 'M') {
                result.Content1 = (byte) 0x83;
                result.Content2 = CalculateAddressStarted(address.substring(1));
            } else if (address.charAt(0) == 'D' || address.substring(0, 2) == "DB") {
                result.Content1 = (byte) 0x84;
                String[] adds = address.split("\\.");
                if (address.charAt(1) == 'B') {
                    result.Content3 = Integer.parseInt(adds[0].substring(2));
                } else {
                    result.Content3 = Integer.parseInt(adds[0].substring(1));
                }

                result.Content2 = CalculateAddressStarted(address.substring(address.indexOf('.') + 1));
            } else if (address.charAt(0) == 'T') {
                result.Content1 = 0x1D;
                result.Content2 = CalculateAddressStarted(address.substring(1));
            } else if (address.charAt(0) == 'C') {
                result.Content1 = 0x1C;
                result.Content2 = CalculateAddressStarted(address.substring(1));
            } else if (address.charAt(0) == 'V') {
                result.Content1 = (byte) 0x84;
                result.Content3 = 1;
                result.Content2 = CalculateAddressStarted(address.substring(1));
            } else {
                return new OperateResultExThree<Byte, Integer, Integer>(StringResources.Language.NotSupportedDataType());
            }
        } catch (Exception ex) {
            result.Message = ex.getMessage();
            return result;
        }

        result.IsSuccess = true;
        return result;
    }

    /**
     * 生成一个读取字数据指令头的通用方法 ->
     * A general method for generating a command header to read a Word data
     *
     * @param address 起始地址，例如M100，I0，Q0，DB2.100 -> Start address, such as M100,I0,Q0,DB2.100
     * @param length  读取数据长度 -> Read Data length
     * @return 包含结果对象的报文 -> Message containing the result object
     */
    public static OperateResultExOne<byte[]> BuildReadCommand(String address, short length) {
        OperateResultExThree<Byte, Integer, Integer> analysis = AnalysisAddress(address);
        if (!analysis.IsSuccess) return OperateResultExOne.<byte[]>CreateFailedResult(analysis);

        OperateResultExThree<Byte, Integer, Integer>[] addressList = new OperateResultExThree[1];
        addressList[0] = analysis;
        short[] lengthList = new short[1];
        lengthList[0] = length;

        return BuildReadCommand(addressList, lengthList);
    }


    /**
     * 生成一个读取字数据指令头的通用方法
     *
     * @param address 解析后的地址
     * @param length  每个地址的读取长度
     * @return 携带有命令字节
     */
    public static OperateResultExOne<byte[]> BuildReadCommand(OperateResultExThree<Byte, Integer, Integer>[] address, short[] length) {
        if (address == null) throw new RuntimeException("address is null");
        if (length == null) throw new RuntimeException("count is null");
        if (address.length != length.length)
            throw new RuntimeException(StringResources.Language.TwoParametersLengthIsNotSame());
        if (length.length > 19)
            throw new RuntimeException(StringResources.Language.SiemensReadLengthCannotLargerThan19());

        int readCount = length.length;

        byte[] _PLCCommand = new byte[19 + readCount * 12];

        // ======================================================================================
        // Header
        // 报文头
        _PLCCommand[0] = 0x03;
        _PLCCommand[1] = 0x00;
        // 长度
        _PLCCommand[2] = (byte) (_PLCCommand.length / 256);
        _PLCCommand[3] = (byte) (_PLCCommand.length % 256);
        // 固定
        _PLCCommand[4] = 0x02;
        _PLCCommand[5] = (byte) 0xF0;
        _PLCCommand[6] = (byte) 0x80;
        // 协议标识
        _PLCCommand[7] = 0x32;
        // 命令：发
        _PLCCommand[8] = 0x01;
        // redundancy identification (reserved): 0x0000;
        _PLCCommand[9] = 0x00;
        _PLCCommand[10] = 0x00;
        // protocol data unit reference; it’s increased by request event;
        _PLCCommand[11] = 0x00;
        _PLCCommand[12] = 0x01;
        // 参数命令数据总长度
        _PLCCommand[13] = (byte) ((_PLCCommand.length - 17) / 256);
        _PLCCommand[14] = (byte) ((_PLCCommand.length - 17) % 256);

        // 读取内部数据时为00，读取CPU型号为Data数据长度
        _PLCCommand[15] = 0x00;
        _PLCCommand[16] = 0x00;


        // ======================================================================================
        // Parameter

        // 读写指令，04读，05写
        _PLCCommand[17] = 0x04;
        // 读取数据块个数
        _PLCCommand[18] = (byte) readCount;


        for (int ii = 0; ii < readCount; ii++) {
            //===========================================================================================
            // 指定有效值类型
            _PLCCommand[19 + ii * 12] = 0x12;
            // 接下来本次地址访问长度
            _PLCCommand[20 + ii * 12] = 0x0A;
            // 语法标记，ANY
            _PLCCommand[21 + ii * 12] = 0x10;
            // 按字为单位
            _PLCCommand[22 + ii * 12] = 0x02;
            // 访问数据的个数
            _PLCCommand[23 + ii * 12] = (byte) (length[ii] / 256);
            _PLCCommand[24 + ii * 12] = (byte) (length[ii] % 256);
            // DB块编号，如果访问的是DB块的话
            _PLCCommand[25 + ii * 12] = (byte) (address[ii].Content3 / 256);
            _PLCCommand[26 + ii * 12] = (byte) (address[ii].Content3 % 256);
            // 访问数据类型
            _PLCCommand[27 + ii * 12] = address[ii].Content1;
            // 偏移位置
            _PLCCommand[28 + ii * 12] = (byte) (address[ii].Content2 / 256 / 256 % 256);
            _PLCCommand[29 + ii * 12] = (byte) (address[ii].Content2 / 256 % 256);
            _PLCCommand[30 + ii * 12] = (byte) (address[ii].Content2 % 256);
        }

        return OperateResultExOne.CreateSuccessResult(_PLCCommand);
    }


    /**
     * 生成一个位读取数据指令头的通用方法
     *
     * @param address 起始地址
     * @return 指令
     */
    public static OperateResultExOne<byte[]> BuildBitReadCommand(String address) {
        byte[] _PLCCommand = new byte[31];
        // 报文头
        _PLCCommand[0] = 0x03;
        _PLCCommand[1] = 0x00;
        // 长度
        _PLCCommand[2] = (byte) (_PLCCommand.length / 256);
        _PLCCommand[3] = (byte) (_PLCCommand.length % 256);
        // 固定
        _PLCCommand[4] = 0x02;
        _PLCCommand[5] = (byte) 0xF0;
        _PLCCommand[6] = (byte) 0x80;
        _PLCCommand[7] = 0x32;
        // 命令：发
        _PLCCommand[8] = 0x01;
        // 标识序列号
        _PLCCommand[9] = 0x00;
        _PLCCommand[10] = 0x00;
        _PLCCommand[11] = 0x00;
        _PLCCommand[12] = 0x01;
        // 命令数据总长度
        _PLCCommand[13] = (byte) ((_PLCCommand.length - 17) / 256);
        _PLCCommand[14] = (byte) ((_PLCCommand.length - 17) % 256);

        _PLCCommand[15] = 0x00;
        _PLCCommand[16] = 0x00;

        // 命令起始符
        _PLCCommand[17] = 0x04;
        // 读取数据块个数
        _PLCCommand[18] = 0x01;


        // 填充数据
        OperateResultExThree<Byte, Integer, Integer> analysis = AnalysisAddress(address);
        if (!analysis.IsSuccess) return OperateResultExOne.CreateFailedResult(analysis);

        //===========================================================================================
        // 读取地址的前缀
        _PLCCommand[19] = 0x12;
        _PLCCommand[20] = 0x0A;
        _PLCCommand[21] = 0x10;
        // 读取的数据时位
        _PLCCommand[22] = 0x01;
        // 访问数据的个数
        _PLCCommand[23] = 0x00;
        _PLCCommand[24] = 0x01;
        // DB块编号，如果访问的是DB块的话
        _PLCCommand[25] = (byte) (analysis.Content3 / 256);
        _PLCCommand[26] = (byte) (analysis.Content3 % 256);
        // 访问数据类型
        _PLCCommand[27] = analysis.Content1;
        // 偏移位置
        _PLCCommand[28] = (byte) (analysis.Content2 / 256 / 256 % 256);
        _PLCCommand[29] = (byte) (analysis.Content2 / 256 % 256);
        _PLCCommand[30] = (byte) (analysis.Content2 % 256);

        return OperateResultExOne.CreateSuccessResult(_PLCCommand);
    }


    /**
     * 生成一个写入字节数据的指令 -> Generate an instruction to write byte data
     *
     * @param address 起始地址，示例M100,I100,Q100,DB1.100 -> Start Address, example M100,I100,Q100,DB1.100
     * @param data    原始的字节数据 -> Raw byte data
     * @return 包含结果对象的报文 -> Message containing the result object
     */
    public static OperateResultExOne<byte[]> BuildWriteByteCommand(String address, byte[] data) {
        if (data == null) data = new byte[0];

        OperateResultExThree<Byte, Integer, Integer> analysis = AnalysisAddress(address);
        if (!analysis.IsSuccess) return OperateResultExOne.CreateFailedResult(analysis);

        return BuildWriteByteCommand(analysis, data);
    }

    /**
     * 生成一个写入字节数据的指令
     *
     * @param analysis 地址
     * @param data     数据
     * @return 指令
     */
    public static OperateResultExOne<byte[]> BuildWriteByteCommand(OperateResultExThree<Byte, Integer, Integer> analysis, byte[] data) {
        if (data == null) data = new byte[0];

        byte[] _PLCCommand = new byte[35 + data.length];
        _PLCCommand[0] = 0x03;
        _PLCCommand[1] = 0x00;
        // 长度
        _PLCCommand[2] = (byte) ((35 + data.length) / 256);
        _PLCCommand[3] = (byte) ((35 + data.length) % 256);
        // 固定
        _PLCCommand[4] = 0x02;
        _PLCCommand[5] = (byte) 0xF0;
        _PLCCommand[6] = (byte) 0x80;
        _PLCCommand[7] = 0x32;
        // 命令 发
        _PLCCommand[8] = 0x01;
        // 标识序列号
        _PLCCommand[9] = 0x00;
        _PLCCommand[10] = 0x00;
        _PLCCommand[11] = 0x00;
        _PLCCommand[12] = 0x01;
        // 固定
        _PLCCommand[13] = 0x00;
        _PLCCommand[14] = 0x0E;
        // 写入长度+4
        _PLCCommand[15] = (byte) ((4 + data.length) / 256);
        _PLCCommand[16] = (byte) ((4 + data.length) % 256);
        // 读写指令
        _PLCCommand[17] = 0x05;
        // 写入数据块个数
        _PLCCommand[18] = 0x01;
        // 固定，返回数据长度
        _PLCCommand[19] = 0x12;
        _PLCCommand[20] = 0x0A;
        _PLCCommand[21] = 0x10;
        // 写入方式，1是按位，2是按字
        _PLCCommand[22] = 0x02;
        // 写入数据的个数
        _PLCCommand[23] = (byte) (data.length / 256);
        _PLCCommand[24] = (byte) (data.length % 256);
        // DB块编号，如果访问的是DB块的话
        _PLCCommand[25] = (byte) (analysis.Content3 / 256);
        _PLCCommand[26] = (byte) (analysis.Content3 % 256);
        // 写入数据的类型
        _PLCCommand[27] = analysis.Content1;
        // 偏移位置
        _PLCCommand[28] = (byte) (analysis.Content2 / 256 / 256 % 256);
        ;
        _PLCCommand[29] = (byte) (analysis.Content2 / 256 % 256);
        _PLCCommand[30] = (byte) (analysis.Content2 % 256);
        // 按字写入
        _PLCCommand[31] = 0x00;
        _PLCCommand[32] = 0x04;
        // 按位计算的长度
        _PLCCommand[33] = (byte) (data.length * 8 / 256);
        _PLCCommand[34] = (byte) (data.length * 8 % 256);

        System.arraycopy(data, 0, _PLCCommand, 35, data.length);

        return OperateResultExOne.CreateSuccessResult(_PLCCommand);
    }


    /**
     * 生成一个写入位数据的指令
     *
     * @param address 起始地址
     * @param data    数据
     * @return 指令
     */
    public static OperateResultExOne<byte[]> BuildWriteBitCommand(String address, boolean data) {
        OperateResultExThree<Byte, Integer, Integer> analysis = AnalysisAddress(address);
        if (!analysis.IsSuccess) return OperateResultExOne.CreateFailedResult(analysis);


        byte[] buffer = new byte[1];
        buffer[0] = data ? (byte) 0x01 : (byte) 0x00;

        byte[] _PLCCommand = new byte[35 + buffer.length];
        _PLCCommand[0] = 0x03;
        _PLCCommand[1] = 0x00;
        // 长度
        _PLCCommand[2] = (byte) ((35 + buffer.length) / 256);
        _PLCCommand[3] = (byte) ((35 + buffer.length) % 256);
        // 固定
        _PLCCommand[4] = 0x02;
        _PLCCommand[5] = (byte) 0xF0;
        _PLCCommand[6] = (byte) 0x80;
        _PLCCommand[7] = 0x32;
        // 命令 发
        _PLCCommand[8] = 0x01;
        // 标识序列号
        _PLCCommand[9] = 0x00;
        _PLCCommand[10] = 0x00;
        _PLCCommand[11] = 0x00;
        _PLCCommand[12] = 0x01;
        // 固定
        _PLCCommand[13] = 0x00;
        _PLCCommand[14] = 0x0E;
        // 写入长度+4
        _PLCCommand[15] = (byte) ((4 + buffer.length) / 256);
        _PLCCommand[16] = (byte) ((4 + buffer.length) % 256);
        // 命令起始符
        _PLCCommand[17] = 0x05;
        // 写入数据块个数
        _PLCCommand[18] = 0x01;
        _PLCCommand[19] = 0x12;
        _PLCCommand[20] = 0x0A;
        _PLCCommand[21] = 0x10;
        // 写入方式，1是按位，2是按字
        _PLCCommand[22] = 0x01;
        // 写入数据的个数
        _PLCCommand[23] = (byte) (buffer.length / 256);
        _PLCCommand[24] = (byte) (buffer.length % 256);
        // DB块编号，如果访问的是DB块的话
        _PLCCommand[25] = (byte) (analysis.Content3 / 256);
        _PLCCommand[26] = (byte) (analysis.Content3 % 256);
        // 写入数据的类型
        _PLCCommand[27] = analysis.Content1;
        // 偏移位置
        _PLCCommand[28] = (byte) (analysis.Content2 / 256 / 256);
        _PLCCommand[29] = (byte) (analysis.Content2 / 256);
        _PLCCommand[30] = (byte) (analysis.Content2 % 256);
        // 按位写入
        _PLCCommand[31] = 0x00;
        _PLCCommand[32] = 0x03;
        // 按位计算的长度
        _PLCCommand[33] = (byte) (buffer.length / 256);
        _PLCCommand[34] = (byte) (buffer.length % 256);

        System.arraycopy(buffer, 0, _PLCCommand, 35, buffer.length);

        return OperateResultExOne.CreateSuccessResult(_PLCCommand);
    }


}
